@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  config_file.c - config file routines                                     */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -42,24 +42,18 @@ gftp_config_vars config_file_vars[] =
         N_("The default directory gFTP will go to on startup")},
   {"logfile", N_("Log file:"), &logfile, CONFIG_CHARTEXT, 1,
 	N_("If this is set, gftp will automagically write a log to this file")},
-  {"connect_timeout", N_("Connect timeout:"), &connect_timeout, CONFIG_INTTEXT,
-        1, N_("The max timeout for the connection")},
-  {"read_timeout", N_("Read timeout:"), &read_timeout, CONFIG_INTTEXT, 1,
-	N_("The timeout for the read()s and write()s")},
-  {"retries", N_("Connect retries:"), &retries, CONFIG_INTTEXT, 1,
-	N_("The number of auto-retries to do. Set this to 0 to retry indefinately")},
-  {"sleep_time", N_("Retry sleep time:"), &sleep_time, CONFIG_INTTEXT, 1,
-	N_("The number of seconds to wait between retries")},
-  {"maxkbs", N_("Max KB/S:"), &maxkbs, CONFIG_FLOATTEXT, 1,
-	N_("The maximum KB/s a file transfer can get. (Set to 0 to disable)")},
+  {"max_log_window_size", N_("Max Log Window Size:"), &max_log_window_size, CONFIG_UINTTEXT,
+        1, N_("The maximum size of the log window in bytes for the GTK+ port")},
   {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"append_transfers", N_("Append file transfers"), &append_file_transfers,
+        CONFIG_CHECKBOX, 1,
+        N_("Append new file transfers onto existing ones")},
   {"reconnect_diag", N_("Bring up reconnect dialog"), &reconnect_diag, 
 	CONFIG_CHECKBOX, 1,
 	N_("Bring up the reconnect dialog after login failure")},
   {"confirm_delete", N_("Confirm delete"), &confirm_delete, CONFIG_CHECKBOX, 1,
 	N_("Confirm when deleting files")},
-  {"one_transfer", N_("Do one transfer at a time"),  &do_one_transfer_at_a_time,
-	CONFIG_CHECKBOX, 1, N_("Do only one transfer at a time?")}, 
+  {"one_transfer", N_("Do one transfer at a time"),  &do_one_transfer_at_a_time, CONFIG_CHECKBOX, 1, N_("Do only one transfer at a time?")}, 
   {"overwrite_default", N_("Overwrite by Default"), &overwrite_by_default,
         CONFIG_CHECKBOX, 1,
         N_("Overwrite files by default or set to resume file transfers")},
@@ -85,11 +79,24 @@ gftp_config_vars config_file_vars[] =
 	N_("Automatically start the file transfers when they get queued?")},
   {"usecache", N_("Use cache"), &use_cache, CONFIG_CHECKBOX, 1,
 	N_("Do you want to use the cache?")},
-  {"use_http11", N_("Use HTTP/1.1"), &use_http11, CONFIG_CHECKBOX, 1,
-	N_("Do you want to use HTTP/1.1 or HTTP/1.0")},
   
+  {"", N_("Network"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
+  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"connect_timeout", N_("Connect timeout:"), &connect_timeout, CONFIG_INTTEXT,
+        1, N_("The max timeout for the connection")},
+  {"read_timeout", N_("Read timeout:"), &read_timeout, CONFIG_INTTEXT, 1,
+	N_("The timeout for the read()s and write()s")},
+  {"retries", N_("Connect retries:"), &retries, CONFIG_INTTEXT, 1,
+	N_("The number of auto-retries to do. Set this to 0 to retry indefinately")},
+  {"sleep_time", N_("Retry sleep time:"), &sleep_time, CONFIG_INTTEXT, 1,
+	N_("The number of seconds to wait between retries")},
+  {"maxkbs", N_("Max KB/S:"), &maxkbs, CONFIG_FLOATTEXT, 1,
+	N_("The maximum KB/s a file transfer can get. (Set to 0 to disable)")},
+  {"", N_("Default Protocol"), (void *) 0x1, CONFIG_COMBO, 1, "DP"},
+  {"default_protocol", N_("Default Protocol"), &default_protocol, 
+        CONFIG_CHARTEXT, 0, N_("This specifies the default protocol to use")},
 
-  {"", N_("FTP Proxy"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
+  {"", N_("FTP"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
   {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
   {"firewall_host", N_("Proxy hostname:"), &firewall_host, CONFIG_CHARTEXT, 1,
 	N_("Firewall hostname")},
@@ -98,11 +105,10 @@ gftp_config_vars config_file_vars[] =
   {"firewall_username", N_("Proxy username:"), &firewall_username,
 	CONFIG_CHARTEXT, 1, N_("Your firewall username")},
   {"firewall_password", N_("Proxy password:"), &firewall_password, 
-	CONFIG_CHARTEXT, 1, N_("Your firewall password")},
-  {"firewall_account", N_("Proxy account:"), &firewall_account, CONFIG_CHARTEXT,
-	1, N_("Your firewall account (optional)")},
-  {"", N_("Proxy server type"), (void *) 0x1, CONFIG_COMBO, 0, ""},
-  {"proxy_config", N_("Proxy config"), &proxy_config, CONFIG_TEXT, 0,
+	CONFIG_CHARPASS, 1, N_("Your firewall password")},
+  {"firewall_account", N_("Proxy account:"), &firewall_account, CONFIG_CHARTEXT, 1, N_("Your firewall account (optional)")},
+  {"", N_("Proxy server type"), (void *) 0x1, CONFIG_COMBO, 1, "PS"},
+  {"proxy_config", N_("Proxy config"), &proxy_config, CONFIG_TEXT, 1,
 	N_("This specifies how your proxy server expects us to log in")},
   {"", N_("%pu = proxy user"), (void *) 0x1, CONFIG_LABEL, 1, ""},
   {"", N_("%hu = host user"), (void *) 0x1, CONFIG_LABEL, 1, ""},
@@ -115,16 +121,40 @@ gftp_config_vars config_file_vars[] =
   {"", N_("%pa = proxy account"), (void *) 0x1, CONFIG_LABEL, 1, ""},
   {"", N_("%ha = host account"), (void *) 0x1, CONFIG_LABEL, 1, ""},
 
-  {"", N_("HTTP Proxy"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
+  {"", N_("HTTP"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
   {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"http_proxy_host", N_("Proxy hostname:"), &http_proxy_host, CONFIG_CHARTEXT, 
+  {"http_proxy_host", N_("Proxy hostname:"), &http_proxy_host, CONFIG_CHARTEXT,
 	1, N_("Firewall hostname")},
   {"http_proxy_port", N_("Proxy port:"), &http_proxy_port, CONFIG_INTTEXT, 1,
 	N_("Port to connect to on the firewall")},
   {"http_proxy_username", N_("Proxy username:"), &http_proxy_username, 
 	CONFIG_CHARTEXT, 1, N_("Your firewall username")},
   {"http_proxy_password", N_("Proxy password:"), &http_proxy_password, 
-	CONFIG_CHARTEXT, 1, N_("Your firewall password")},
+	CONFIG_CHARPASS, 1, N_("Your firewall password")},
+  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"use_http11", N_("Use HTTP/1.1"), &use_http11, CONFIG_CHECKBOX, 1,
+	N_("Do you want to use HTTP/1.1 or HTTP/1.0")},
+
+  {"", N_("SSH"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
+  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"ssh_prog_name", N_("SSH Prog Name:"), &ssh_prog_name, CONFIG_CHARTEXT, 1,
+	N_("The path to the SSH executable")},
+  {"ssh_extra_params", N_("SSH Extra Params:"), &ssh_extra_params, 
+        CONFIG_CHARTEXT, 1, N_("Extra parameters to pass to the SSH program")},
+  {"ssh1_sftp_path", N_("SSH sftpserv path:"), &ssh1_sftp_path,
+	CONFIG_CHARTEXT, 1, N_("Default remote SSH sftpserv path")},
+  {"ssh2_sftp_path", N_("SSH2 sftp-server path:"), &ssh2_sftp_path,
+	CONFIG_CHARTEXT, 1, N_("Default remote SSH2 sftp-server path")},
+  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"ssh_need_userpass", N_("Need SSH User/Pass"), &ssh_need_userpass,
+        CONFIG_CHECKBOX, 1,
+        N_("Require a username/password for SSH connections")},
+  {"ssh_use_askpass", N_("Use ssh-askpass util"), &ssh_use_askpass,
+        CONFIG_CHECKBOX, 1,
+        N_("Use the ssh-askpass utility to grab the users password")},
+  {"sshv2_use_sftp_subsys", N_("Use SSH2 SFTP subsys"), 
+        &sshv2_use_sftp_subsys, CONFIG_CHECKBOX, 1,
+        N_("Call ssh with the -s sftp flag. This is helpful because you won't have to know the remote path to the remote sftp-server")},
 
   {"list_dblclk_action", "", &listbox_dblclick_action, CONFIG_HIDEINT, 0,
 	N_("This defines what will happen when you double click a file in the file listboxes. 0=View file 1=Edit file 2=Transfer file")},
@@ -166,6 +196,14 @@ gftp_config_vars config_file_vars[] =
 	N_("The width of the date column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
   {"remote_attribs_width", "", &remote_columns[5], CONFIG_HIDEINT, 0,
 	N_("The width of the attribs column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
+  {"send_color", "", &send_color, CONFIG_COLOR, 0,
+        N_("The color of the commands that are sent to the server")},
+  {"recv_color", "", &recv_color, CONFIG_COLOR, 0,
+        N_("The color of the commands that are received from the server")},
+  {"error_color", "", &error_color, CONFIG_COLOR, 0,
+        N_("The color of the error messages")},
+  {"misc_color", "", &misc_color, CONFIG_COLOR, 0,
+        N_("The color of the rest of the log messages")},
   {NULL, NULL, NULL, 0, 0, ""}
 };
 
@@ -173,19 +211,32 @@ gftp_config_vars config_file_vars[] =
 void
 gftp_read_config_file (int get_xpms)
 {
-  char *tempstr, *temp1str, *curpos, buf[255], *str;
+  char *tempstr, *temp1str, *curpos, *endpos, buf[255], *str, *red, *green, 
+       *blue;
   gftp_file_extensions * tempext;
   gftp_bookmarks * newentry;
   gftp_proxy_hosts * host;
   unsigned int nums[4];
   struct hostent *hent;
   struct utsname unme;
+  gftp_color * color;
   struct passwd *pw;
   FILE *conffile;
+  uid_t my_uid;
+  gid_t my_gid;
   int line, i;
   size_t len;
   void *ptr;
 
+  memset (&send_color, 0, sizeof (send_color));
+  memset (&recv_color, 0, sizeof (recv_color));
+  memset (&error_color, 0, sizeof (error_color));
+  memset (&misc_color, 0, sizeof (misc_color));
+  send_color.green = 0x8600;
+  recv_color.blue = 0xffff;
+  error_color.red = 0xffff;
+  misc_color.red = 0xffff;
+
   line = 0;
   bookmarks = g_malloc0 (sizeof (*bookmarks));
   bookmarks->isfolder = 1;
@@ -234,7 +285,8 @@ gftp_read_config_file (int get_xpms)
  
   for (i = 0; config_file_vars[i].var != NULL; i++)
     {
-      g_hash_table_insert (config_htable, config_file_vars[i].key, (void *) i);
+      g_hash_table_insert (config_htable, config_file_vars[i].key, 
+                           GINT_TO_POINTER(i));
     } 
 
   while (fgets (buf, sizeof (buf), conffile))
@@ -254,9 +306,10 @@ gftp_read_config_file (int get_xpms)
 
       if ((ptr = g_hash_table_lookup (config_htable, buf)) != NULL)
         {
-          i = (int) ptr;
+          i = GPOINTER_TO_INT(ptr);
           curpos = buf + strlen (buf) + 1;
           if (config_file_vars[i].type == CONFIG_CHARTEXT ||
+              config_file_vars[i].type == CONFIG_CHARPASS ||
               config_file_vars[i].type == CONFIG_TEXT)
             {
               *(char **) config_file_vars[i].var = g_malloc (strlen (curpos) + 1);
@@ -264,6 +317,17 @@ gftp_read_config_file (int get_xpms)
             }
           else if (config_file_vars[i].type == CONFIG_FLOATTEXT)
             *(double *) config_file_vars[i].var = strtod (curpos, NULL);
+          else if (config_file_vars[i].type == CONFIG_COLOR)
+            {
+              color = config_file_vars[i].var;
+              parse_args (curpos, 3, line, &red, &green, &blue);
+              color->red = strtol (red, NULL, 16);
+              color->green = strtol (green, NULL, 16);
+              color->blue = strtol (blue, NULL, 16);
+              g_free (red);
+              g_free (green);
+              g_free (blue);
+            }
           else
             *(int *) config_file_vars[i].var = strtol (curpos, NULL, 10);
 	}
@@ -295,7 +359,7 @@ gftp_read_config_file (int get_xpms)
 	  else
 	    {
 	      *tempstr = '\0';
-	      sscanf (curpos, "%d.%d.%d.%d", &nums[0], &nums[1], &nums[2],
+	      sscanf (curpos, "%u.%u.%u.%u", &nums[0], &nums[1], &nums[2],
 		      &nums[3]);
 	      host->ipv4_network_address =
 		nums[0] << 24 | nums[1] << 16 | nums[2] << 8 | nums[3];
@@ -305,7 +369,7 @@ gftp_read_config_file (int get_xpms)
 		    0xffffffff << (32 - strtol (tempstr + 1, NULL, 10));
 	      else
 		{
-		  sscanf (tempstr + 1, "%d.%d.%d.%d", &nums[0], &nums[1],
+		  sscanf (tempstr + 1, "%u.%u.%u.%u", &nums[0], &nums[1],
 			  &nums[2], &nums[3]);
 		  host->ipv4_netmask =
 		    nums[0] << 24 | nums[1] << 16 | nums[2] << 8 | nums[3];
@@ -382,6 +446,12 @@ gftp_read_config_file (int get_xpms)
       strcpy (logfile, LOG_FILE);
     }
 
+  if (default_protocol == NULL)
+    {
+      default_protocol = g_malloc (4);
+      strcpy (default_protocol, "FTP");
+    }
+
   if (*logfile != '\0')
     {
       if ((tempstr = expand_path (logfile)) == NULL)
@@ -400,6 +470,36 @@ gftp_read_config_file (int get_xpms)
 
   gftp_read_bookmarks ();
 
+  if (ssh_extra_params != NULL)
+    {
+      num_ssh_extra_params = 0;
+      curpos = ssh_extra_params;
+      while ((endpos = strchr (curpos, ' ')) != NULL)
+        {
+          *endpos = '\0';
+          num_ssh_extra_params++;
+          ssh_extra_params_list = g_realloc (ssh_extra_params_list,
+                                             (num_ssh_extra_params + 1) * 
+                                             sizeof (char *));
+          ssh_extra_params_list[num_ssh_extra_params - 1] = 
+                                                  g_strconcat (curpos, NULL);
+          ssh_extra_params_list[num_ssh_extra_params] = NULL;
+          *endpos = ' ';
+          curpos = endpos + 1;
+        }
+
+      if (*curpos != '\0')
+        {
+          num_ssh_extra_params++;
+          ssh_extra_params_list = g_realloc (ssh_extra_params_list,
+                                             (num_ssh_extra_params + 1) * 
+                                             sizeof (char *));
+          ssh_extra_params_list[num_ssh_extra_params - 1] = 
+                                                  g_strconcat (curpos, NULL);
+          ssh_extra_params_list[num_ssh_extra_params] = NULL;
+        }
+    }
+
   make_nonnull (&proxy_config);
   make_nonnull (&firewall_host);
   make_nonnull (&firewall_username);
@@ -411,7 +511,9 @@ gftp_read_config_file (int get_xpms)
   make_nonnull (&view_program);
   make_nonnull (&edit_program);
 
-  pw = getpwuid (geteuid ());
+  my_uid = geteuid ();
+  my_gid = getegid ();
+  pw = getpwuid (my_uid);
   if (emailaddr == NULL || *emailaddr == '\0')
     {
       /* If there is no email address specified, then we'll just use the
@@ -428,16 +530,9 @@ gftp_read_config_file (int get_xpms)
     }
 
   if ((tempstr = getenv ("TMP")) != NULL && *tempstr != '\0')
-    tmp_directory = g_strconcat (tempstr, "/.gftp-", pw->pw_name, NULL);
+    tmp_directory = g_strconcat (tempstr, NULL);
   else
-    tmp_directory = g_strconcat ("/tmp/.gftp-", pw->pw_name, NULL);
-
-  if (access (tmp_directory, F_OK) == -1 && mkdir (tmp_directory, 0x1C0) != 0)
-    {
-      printf (_("gFTP Error: Cannot make temp directory %s: %s\n"), 
-              tmp_directory, g_strerror (errno));
-      exit (1);
-    }
+    tmp_directory = g_strconcat ("/tmp", NULL);
 }
 
 
@@ -569,6 +664,14 @@ gftp_read_bookmarks (void)
 	  newentry->acct = g_malloc (strlen (curpos) + 1);
 	  strcpy (newentry->acct, curpos);
 	}
+      else if (strncmp (buf, "sftpserv_path", 13) == 0 && newentry)
+        {
+          curpos = buf + 14;
+          if (newentry->sftpserv_path)
+            g_free (newentry->sftpserv_path);
+          newentry->sftpserv_path = g_malloc (strlen (curpos) + 1);
+          strcpy (newentry->sftpserv_path, curpos);
+        }
       else if (*buf != '#' && *buf != '\0')
 	printf (_("gFTP Warning: Skipping line %d in bookmarks file: %s\n"),
 		line, buf);
@@ -674,11 +777,12 @@ gftp_write_config_file (void)
   gftp_proxy_hosts *hosts;
   FILE *conffile, *bmfile;
   char *tempstr, *str;
+  gftp_color *color;
   GList *templist;
   int pos;
 
-  hdr = N_("# Config file for gFTP\n# Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>\n# Warning: Any comments that you add to this file WILL be overwritten\n# If a entry has a (*) in it's comment, you can't change it inside gFTP\n\n");
-  bmhdr = N_("# Bookmarks file for gFTP\n# Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>\n# Warning: Any comments that you add to this file WILL be overwritten\n\n");
+  hdr = N_("Config file for gFTP. Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>. Warning: Any comments that you add to this file WILL be overwritten. If a entry has a (*) in it's comment, you can't change it inside gFTP");
+  bmhdr = N_("Bookmarks file for gFTP. Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>. Warning: Any comments that you add to this file WILL be overwritten");
   proxyhdr = N_("This section specifies which hosts are on the local subnet and won't need to go out the proxy server (if available). Syntax: dont_use_proxy=.domain or dont_use_proxy=network number/netmask");
   exthdr = N_("ext=file extenstion:XPM file:Ascii or Binary (A or B):viewer program. Note: All arguments except the file extension are optional");
   histhdr = N_("This section contains the data that is in the history");
@@ -696,7 +800,8 @@ gftp_write_config_file (void)
     }
   g_free (tempstr);
 
-  fwrite (_(hdr), 1, strlen (_(hdr)), conffile);
+  write_comment (conffile, _(hdr));
+  fwrite ("\n", 1, 1, conffile);
 
   if ((tempstr = expand_path (BOOKMARKS_FILE)) == NULL)
     {
@@ -711,11 +816,13 @@ gftp_write_config_file (void)
     }
   g_free (tempstr);
 
-  fwrite (_(bmhdr), 1, strlen (_(bmhdr)), bmfile);
+  write_comment (bmfile, _(bmhdr));
+  fwrite ("\n", 1, 1, bmfile);
 
   for (pos = 0; config_file_vars[pos].var != NULL; pos++)
     {
       if (config_file_vars[pos].type == CONFIG_CHARTEXT ||
+          config_file_vars[pos].type == CONFIG_CHARPASS ||
           config_file_vars[pos].type == CONFIG_TEXT)
         {
           if (*config_file_vars[pos].comment != '\0')
@@ -734,6 +841,7 @@ gftp_write_config_file (void)
                    *(double *) config_file_vars[pos].var);
         }
       else if (config_file_vars[pos].type == CONFIG_INTTEXT ||
+               config_file_vars[pos].type == CONFIG_UINTTEXT ||
                config_file_vars[pos].type == CONFIG_CHECKBOX ||
                config_file_vars[pos].type == CONFIG_HIDEINT)
         {
@@ -743,6 +851,15 @@ gftp_write_config_file (void)
 	  fprintf (conffile, "%s=%d\n\n", config_file_vars[pos].key,
                    *(int *) config_file_vars[pos].var);
         }
+      else if (config_file_vars[pos].type == CONFIG_COLOR)
+        {
+          if (*config_file_vars[pos].comment != '\0')
+            write_comment (conffile, _(config_file_vars[pos].comment));
+
+          color = config_file_vars[pos].var;
+          fprintf (conffile, "%s=%x:%x:%x\n\n", config_file_vars[pos].key,
+                   color->red, color->green, color->blue);
+        }
     }
 
   tempentry = bookmarks->children;
@@ -757,7 +874,7 @@ gftp_write_config_file (void)
       while (*str == '/')
 	str++;
       fprintf (bmfile,
-	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n\n",
+	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
 	       str, tempentry->hostname == NULL ? "" : tempentry->hostname,
 	       tempentry->port, tempentry->protocol == NULL
 	       || *tempentry->protocol ==
@@ -769,6 +886,11 @@ gftp_write_config_file (void)
 	       || tempentry->pass == NULL ? "" : tempentry->pass,
 	       tempentry->acct == NULL ? "" : tempentry->acct);
 
+      if (tempentry->sftpserv_path)
+        fprintf (bmfile, "sftpserv_path=%s\n", tempentry->sftpserv_path);
+
+      fprintf (bmfile, "\n");
+ 
       if (tempentry->next == NULL)
 	{
 	  tempentry = tempentry->prev;
@@ -838,6 +960,9 @@ write_comment (FILE * fd, const char *comment)
 {
   const char *pos, *endpos;
 
+  /* FIXME - need to use UTF functions here. I need to check to see how 
+     portable they are before I do this though */
+
   fwrite ("# ", 1, 2, fd);
   pos = comment;
   while (strlen (pos) > 76)
