@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp.h - include file for the whole ftp program                          */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -28,7 +28,6 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/time.h>
-#include <sys/poll.h>
 #ifndef TIOCGWINSZ
 #include <sys/ioctl.h>
 #endif
@@ -69,6 +68,10 @@
 #endif
 #define N_(String) String
 
+#ifndef AF_LOCAL
+#define AF_LOCAL AF_UNIX
+#endif
+
 /* Some general settings */
 #define BASE_CONF_DIR		"~/.gftp"
 #define CONFIG_FILE		BASE_CONF_DIR "/gftprc"
@@ -76,7 +79,6 @@
 #define LOG_FILE		BASE_CONF_DIR "/gftp.log"
 #define MAX_HIST_LEN		10
 
-
 typedef enum gftp_transfer_type_tag 
 {
   gftp_transfer_passive,
@@ -131,9 +133,11 @@ struct gftp_file_tag
                done_view : 1,	/* View the file when done transfering? */
                done_edit : 1,	/* Edit the file when done transfering? */
                done_rm : 1,	/* Remove the file when done */
-               transfer_done : 1; /* Is current file transfer done? */
+               transfer_done : 1, /* Is current file transfer done? */
+               is_fd : 1;	/* Is this a file descriptor? */
   char transfer_action;		/* See the GFTP_TRANS_ACTION_* vars above */
   void *node;			/* Pointer to the node for the gui */
+  FILE * fd;
 };
 
 
@@ -177,15 +181,13 @@ struct gftp_request_tag
   int wakeup_main_thread[2];	/* FD that gets written to by the threads
                                    to wakeup the parent */
         
-#ifdef HAVE_GETADDRINFO
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
   struct addrinfo *hostp;	/* Remote host we are connected to */
 #else
   struct hostent host, *hostp;  /* Remote host we are connected to */
 #endif
 
-
   int data_type;		/* ASCII or BINARY (FTP only) */
-  gftp_transfer_type transfer_type;	/* Passive or non-passive (FTP only) */
   unsigned int use_proxy : 1,           /* Go out of proxy server */
                always_connected : 1,
                need_hostport : 1,
@@ -196,7 +198,7 @@ struct gftp_request_tag
                cached : 1,              /* Is this directory listing cached? */
                stopable : 1;
 
-  long gotbytes;
+  off_t gotbytes;
  
   void *protocol_data;
    
@@ -207,19 +209,21 @@ struct gftp_request_tag
   void (*destroy)			( gftp_request * request );
   int (*connect)			( gftp_request * request );
   void (*disconnect) 			( gftp_request * request );
-  long (*get_file) 			( gftp_request * request, 
+  off_t (*get_file) 			( gftp_request * request, 
 					  const char *filename, 
-					  long startsize );
+					  FILE * fd,
+					  off_t startsize );
   int (*put_file) 			( gftp_request * request, 
 					  const char *filename, 
-					  long startsize,
-					  long totalsize );
-  long (*transfer_file) 		( gftp_request * fromreq, 
+					  FILE * fd,
+					  off_t startsize,
+					  off_t totalsize );
+  off_t (*transfer_file) 		( gftp_request * fromreq, 
 					  const char *fromfile, 
-					  long fromsize, 
+					  off_t fromsize, 
 					  gftp_request * toreq, 
 					  const char *tofile, 
-					  long tosize );
+					  off_t tosize );
   size_t (*get_next_file_chunk) 	( gftp_request * request, 
 					  char *buf, 
 					  size_t size );
@@ -233,7 +237,7 @@ struct gftp_request_tag
 					  FILE *fd );
   int (*set_data_type)			( gftp_request * request, 
 					  int data_type );
-  long (*get_file_size) 		( gftp_request * request, 
+  off_t (*get_file_size) 		( gftp_request * request, 
 					  const char *filename );
   int (*chdir)				( gftp_request * request, 
 					  const char *directory );
@@ -254,6 +258,16 @@ struct gftp_request_tag
 					  time_t datettime );
   int (*site)				( gftp_request * request, 
 					  const char *filename );
+
+  /* Options */
+  gftp_transfer_type transfer_type;	/* Passive or non-passive (FTP only) */
+  int connect_timeout,
+      read_timeout,
+      retries,
+      sleep_time,
+      passive_transfer;
+  float maxkbs;
+  char *sftpserv_path;
 };
 
 
@@ -288,14 +302,16 @@ struct gftp_transfer_tag
   long numfiles,
        numdirs;
 
-  unsigned long curtrans,
-                trans_bytes,
-                total_bytes;
+  off_t curtrans,
+        trans_bytes,
+        total_bytes,
+        resumed_bytes;
 
   void * fromwdata,
        * towdata;
 
-  void * mutex;
+  void *statmutex,
+       *structmutex;
   void *node;
   void *clist;
 };
@@ -337,6 +353,9 @@ struct gftp_bookmarks_tag
                  *prev, 	/* The parent of this node */
                  *next; 	/* The next sibling of this node */
   gpointer cnode; 
+
+  /* Site options */
+  char *sftpserv_path;		/* Path to the sftp server */
 };
 
 
@@ -350,6 +369,14 @@ typedef struct gftp_file_extensions_tag
 } gftp_file_extensions;
 
 
+typedef struct gftp_color_tag
+{
+  gushort red,
+          green,
+          blue;
+} gftp_color;
+
+
 typedef struct gftp_config_vars_tag
 {
   char *key,			/* variable name */
@@ -371,6 +398,9 @@ typedef struct gftp_config_vars_tag
 #define CONFIG_CHARTEXT                 8
 #define CONFIG_COMBO			9
 #define CONFIG_TEXT			10
+#define CONFIG_COLOR			11
+#define CONFIG_UINTTEXT                 12
+#define CONFIG_CHARPASS			13
 
 typedef struct gftp_proxy_type_tag
 {
@@ -382,10 +412,14 @@ typedef struct gftp_proxy_type_tag
 
 /* Global config options */
 extern supported_gftp_protocols gftp_protocols[];
-extern char version[], *emailaddr, *edit_program, *view_program, *firewall_host,
-            *firewall_username, *firewall_password, *firewall_account,
-            *proxy_config, *http_proxy_host, *http_proxy_username, 
-            *http_proxy_password, *logfile, *tmp_directory, *startup_directory;
+extern char version[], *emailaddr, *edit_program, *view_program, 
+            *firewall_host, *firewall_username, *firewall_password, 
+            *firewall_account, *proxy_config, *http_proxy_host, 
+            *http_proxy_username, *http_proxy_password, *logfile, 
+            *tmp_directory, *startup_directory, *ssh_prog_name, 
+            *ssh_extra_params, **ssh_extra_params_list, *default_protocol,
+            *ssh1_sftp_path, *ssh2_sftp_path;
+extern int num_ssh_extra_params;
 extern FILE * logfd;
 extern double maxkbs;
 extern GList * proxy_hosts, * registered_exts, * viewedit_processes, 
@@ -400,15 +434,18 @@ extern int do_one_transfer_at_a_time, start_file_transfers,
            log_height, connect_timeout, retries, sleep_time, read_timeout, 
            use_http11, listbox_dblclick_action, file_trans_column, 
            local_columns[6], remote_columns[6], resolve_symlinks, 
-           firewall_port, http_proxy_port, overwrite_by_default;
+           firewall_port, http_proxy_port, overwrite_by_default,
+           append_file_transfers, ssh_need_userpass, ssh_use_askpass,
+           sshv2_use_sftp_subsys;
+extern guint max_log_window_size;
 extern GHashTable * bookmarks_htable, * config_htable;
 extern GList * localhistory, * remotehistory, * host_history, * port_history, 
              * user_history;
 extern unsigned int host_len, port_len, user_len, localhistlen, remotehistlen;
 extern volatile sig_atomic_t viewedit_process_done;
-extern sigjmp_buf envir;
 extern gftp_config_vars config_file_vars[];
 extern gftp_proxy_type proxy_type[];
+extern gftp_color send_color, recv_color, error_color, misc_color;
 
 /* cache.c */
 FILE * gftp_new_cache_entry 		( gftp_request * request );
@@ -442,7 +479,7 @@ long file_countlf 			( int filefd,
 
 char *alltrim 				( char *str );
 
-char *expand_path 			( char *src );
+char *expand_path 			( const char *src );
 
 void remove_double_slashes 		( char *string );
 
@@ -484,9 +521,24 @@ void free_tdata 			( gftp_transfer * tdata );
 
 gftp_request * copy_request 		( gftp_request * req );
 
-void sig_quit 				( int signo );
+int ptym_open 				( char *pts_name );
 
-char *make_temp_filename 		( char *ext );
+int ptys_open 				( int fdm, 
+					  char *pts_name );
+
+int tty_raw 				( int fd );
+
+char **make_ssh_exec_args 		( gftp_request * request, 
+					  char *execname,
+					  int use_sftp_subsys,
+					  char *portstring );
+
+char * ssh_start_login_sequence 	( gftp_request * request, 
+					  int fd );
+
+#ifdef G_HAVE_GINT64
+gint64 hton64				( gint64 val );
+#endif
 
 /* protocols.c */
 #define GFTP_CONNECTED(request)			(request->sockfd != NULL)
@@ -519,6 +571,7 @@ char *make_temp_filename 		( char *ext );
 #define GFTP_HTTP_NUM				1
 #define GFTP_LOCAL_NUM				2
 #define GFTP_SSH_NUM				3
+#define GFTP_SSHV2_NUM				4
 #define GFTP_TYPE_BINARY			1
 #define GFTP_TYPE_ASCII				2   
 #define GFTP_IS_CONNECTED(request)		((request) != NULL && \
@@ -538,6 +591,8 @@ void local_init 			( gftp_request * request );
 
 void ssh_init 				( gftp_request * request );
 
+void sshv2_init 			( gftp_request * request );
+
 gftp_request *gftp_request_new 		( void );
 
 void gftp_request_destroy 		( gftp_request * request );
@@ -548,21 +603,25 @@ int gftp_connect 			( gftp_request * request );
 
 void gftp_disconnect 			( gftp_request * request );
 
-long gftp_get_file 			( gftp_request * request, 
+size_t gftp_get_file 			( gftp_request * request, 
 					  const char *filename, 
-					  long startsize );
+					  FILE * fd,
+					  size_t startsize );
 
 int gftp_put_file 			( gftp_request * request, 
 					  const char *filename, 
-					  long startsize,
-					  long totalsize );
+					  FILE * fd,
+					  size_t startsize,
+					  size_t totalsize );
 
 long gftp_transfer_file 		( gftp_request *fromreq, 
 					  const char *fromfile, 
-					  long fromsize, 
+					  FILE * fromfd,
+					  size_t fromsize, 
 					  gftp_request *toreq, 
 					  const char *tofile, 
-					  long tosize );
+					  FILE * tofd,
+					  size_t tosize );
 
 size_t gftp_get_next_file_chunk 	( gftp_request * request, 
 					  char *buf, 
@@ -674,7 +733,10 @@ int gftp_get_file_transfer_mode 	( char *filename,
 int gftp_connect_server 		( gftp_request * request, 
 					  char *service );
 
-#ifdef HAVE_GETADDRINFO
+void gftp_set_sftpserv_path		( gftp_request * request,
+					  char *path );
+
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
 
 int get_port 				( struct addrinfo *addr );
 
@@ -690,6 +752,8 @@ struct servent *r_getservbyname 	( const char *name,
 					  int *h_errnop );
 #endif
 
+void gftp_set_config_options 		( gftp_request * request );
+
 void print_file_list 			( GList * list );
 
 #endif
