@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  local.c - functions that will use the local system                       */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -25,19 +25,21 @@ static void local_remove_key 			( gpointer key,
 						  gpointer user_data );
 static int local_connect 			( gftp_request * request );
 static void local_disconnect 			( gftp_request * request );
-static long local_get_file 			( gftp_request * request, 
+static off_t local_get_file 			( gftp_request * request, 
 						  const char *filename,
-						  long startsize );
+						  FILE * fd,
+						  off_t startsize );
 static int local_put_file 			( gftp_request * request, 
 						  const char *filename,
-						  long startsize,
-						  long totalsize );
+						  FILE * fd,
+						  off_t startsize,
+						  off_t totalsize );
 static int local_end_transfer 			( gftp_request * request );
 static int local_get_next_file 			( gftp_request * request, 
 						  gftp_file * fle, 
 						  FILE * fd );
 static int local_list_files 			( gftp_request * request );
-static long local_get_file_size 		( gftp_request * request,
+static off_t local_get_file_size 		( gftp_request * request,
 						  const char *filename );
 static int local_chdir 				( gftp_request * request, 
 						  const char *directory );
@@ -183,28 +185,40 @@ local_disconnect (gftp_request * request)
 
   if (request->datafd)
     {
-      fclose (request->datafd);
+      if (fclose (request->datafd) < 0)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                     _("Error closing file descriptor: %s\n"),
+                                     g_strerror (errno));
+        }
       request->datafd = NULL;
     }
+  request->sockfd = NULL;
 }
 
 
-static long
-local_get_file (gftp_request * request, const char *filename, long startsize)
+static off_t 
+local_get_file (gftp_request * request, const char *filename, FILE * fd,
+                off_t startsize)
 {
-  long size;
+  size_t size;
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
   g_return_val_if_fail (filename != NULL, -2);
 
-  if ((request->datafd = fopen (filename, "rb")) == NULL)
+  if (fd == NULL)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Error: Cannot open local file %s: %s\n"),
-                                 filename, g_strerror (errno));
-      return (-2);
+      if ((request->datafd = fopen (filename, "rb")) == NULL)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                   _("Error: Cannot open local file %s: %s\n"),
+                                   filename, g_strerror (errno));
+          return (-2);
+        }
     }
+  else
+    request->datafd = fd;
 
   if (lseek (fileno (request->datafd), 0, SEEK_END) == -1)
     {
@@ -238,8 +252,8 @@ local_get_file (gftp_request * request, const char *filename, long startsize)
 
 
 static int
-local_put_file (gftp_request * request, const char *filename, long startsize,
-                long totalsize)
+local_put_file (gftp_request * request, const char *filename, FILE * fd,
+                off_t startsize, off_t totalsize)
 {
   int sock;
 
@@ -247,33 +261,38 @@ local_put_file (gftp_request * request, const char *filename, long startsize,
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
   g_return_val_if_fail (filename != NULL, -2);
 
-  if ((sock = open (filename, 
-      startsize > 0 ? O_WRONLY | O_APPEND | O_CREAT: O_WRONLY | O_CREAT, 
-      S_IRUSR | S_IWUSR)) < 0)
+  if (fd == NULL)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Error: Cannot open local file %s: %s\n"),
-                                 filename, g_strerror (errno));
-      return (-2);
-    }
+      if ((sock = open (filename, 
+          startsize > 0 ? O_WRONLY | O_APPEND | O_CREAT: O_WRONLY | O_CREAT, 
+          S_IRUSR | S_IWUSR)) < 0)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                   _("Error: Cannot open local file %s: %s\n"),
+                                   filename, g_strerror (errno));
+          return (-2);
+        }
 
-  if (ftruncate (sock, startsize) == -1)
+      if ((request->datafd = fdopen (sock, "ab")) == NULL)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                     _("Cannot fdopen() socket for %s: %s\n"),
+                                     filename, g_strerror (errno));
+          close (sock);
+          return (-2);
+        }
+    }
+  else
+    request->datafd = fd;
+
+  if (ftruncate (fileno (request->datafd), startsize) == -1)
     {
       request->logging_function (gftp_logging_error, request->user_data,
-                                _("Error: Cannot truncate local file %s: %s\n"),
-                                filename, g_strerror (errno));
+                               _("Error: Cannot truncate local file %s: %s\n"),
+                               filename, g_strerror (errno));
       return (-2);
     }
     
-  if ((request->datafd = fdopen (sock, "ab")) == NULL)
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Cannot fdopen() socket for %s: %s\n"),
-                                 filename, g_strerror (errno));
-      close (sock);
-      return (-2);
-    }
-
   if (fseek (request->datafd, startsize, SEEK_SET) == -1)
     {
       request->logging_function (gftp_logging_error, request->user_data,
@@ -300,7 +319,12 @@ local_end_transfer (gftp_request * request)
 
   if (request->datafd)
     {
-      fclose (request->datafd);
+      if (fclose (request->datafd) < 0)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                     _("Error closing file descriptor: %s\n"),
+                                     g_strerror (errno));
+        }
       request->datafd = NULL;
     }
 
@@ -344,15 +368,17 @@ local_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
       return (-2);
     }
 
-  if ((user = g_hash_table_lookup (lpd->userhash, (gpointer) st.st_uid)) != NULL)
+  if ((user = g_hash_table_lookup (lpd->userhash, 
+                                   GUINT_TO_POINTER(st.st_uid))) != NULL)
     {
       fle->user = g_malloc (strlen (user) + 1);
       strcpy (fle->user, user);
     }
   else
     {
+      /* FIXME - IRIX complains */
       if ((pw = getpwuid (st.st_uid)) == NULL)
-        fle->user = g_strdup_printf ("%u", st.st_uid);
+        fle->user = g_strdup_printf ("%u", st.st_uid); 
       else
         {
           fle->user = g_malloc (strlen (pw->pw_name) + 1);
@@ -364,15 +390,17 @@ local_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
       g_hash_table_insert (lpd->userhash, GUINT_TO_POINTER (st.st_uid), user);
     }
 
-  if ((group = g_hash_table_lookup (lpd->grouphash, (gpointer) st.st_gid)) != NULL)
+  if ((group = g_hash_table_lookup (lpd->grouphash, 
+                                    GUINT_TO_POINTER(st.st_gid))) != NULL)
     {
       fle->group = g_malloc (strlen (group) + 1);
       strcpy (fle->group, group);
     }
   else
     {
+      /* FIXME - IRIX complains */
       if ((gr = getgrgid (st.st_gid)) == NULL)
-        fle->group = g_strdup_printf ("%u", st.st_gid);
+        fle->group = g_strdup_printf ("%u", st.st_gid); 
       else
         {
           fle->group = g_malloc (strlen (gr->gr_name) + 1);
@@ -432,7 +460,7 @@ local_list_files (gftp_request * request)
 }
 
 
-static long
+static off_t 
 local_get_file_size (gftp_request * request, const char *filename)
 {
   struct stat st;
