@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  misc.c - general purpose routines                                        */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -138,7 +138,7 @@ alltrim (char *str)
 
 
 char *
-expand_path (char *src)
+expand_path (const char *src)
 {
   char *str, *pos, *endpos, *prevpos, *newstr, *tempstr, tempchar;
   struct passwd *pw;
@@ -284,8 +284,15 @@ copyfile (char *source, char *dest)
   while ((n = fread (buf, 1, sizeof (buf), srcfd)) > 0)
     fwrite (buf, 1, n, destfd);
 
-  fclose (srcfd);
-  fclose (destfd);
+  if (fclose (srcfd) < 0)
+    {
+      fclose (destfd);
+      return (0);
+    }
+
+  if (fclose (destfd) < 0)
+    return (0);
+
   return (1);
 }
 
@@ -340,9 +347,9 @@ gftp_parse_command_line (int *argc, char ***argv)
 {
   if (*argc > 1)
     {
-      if (strcmp (**argv, "--help") == 0 || strcmp (**argv, "-h") == 0)
+      if (strcmp (argv[0][1], "--help") == 0 || strcmp (argv[0][1], "-h") == 0)
 	gftp_usage ();
-      else if (strcmp (**argv, "--version") == 0 || strcmp (**argv, "-v") == 0)
+      else if (strcmp (argv[0][1], "--version") == 0 || strcmp (argv[0][1], "-v") == 0)
 	{
 	  printf ("%s\n", version);
 	  exit (0);
@@ -437,6 +444,8 @@ free_fdata (gftp_file * fle)
     g_free (fle->attribs);
   if (fle->destfile)
     g_free (fle->destfile);
+  if (fle->fd)
+    free (fle->fd);
   g_free (fle);
 }
 
@@ -548,8 +557,10 @@ compare_request (gftp_request * request1, gftp_request * request2,
 void
 free_tdata (gftp_transfer * tdata)
 {
-  if (tdata->mutex)
-    g_free (tdata->mutex);
+  if (tdata->statmutex)
+    g_free (tdata->statmutex);
+  if (tdata->structmutex)
+    g_free (tdata->structmutex);
   if (tdata->fromreq != NULL)
     gftp_request_destroy (tdata->fromreq);
   if (tdata->toreq != NULL)
@@ -591,6 +602,8 @@ copy_request (gftp_request * req)
     newreq->url_prefix = g_strconcat (req->url_prefix, NULL);
   if (req->protocol_name)
     newreq->protocol_name = g_strconcat (req->protocol_name, NULL);
+  if (req->sftpserv_path)
+    newreq->sftpserv_path = g_strconcat (req->sftpserv_path, NULL);
 
   newreq->last_ftp_response = NULL;
   newreq->last_dir_entry = NULL;
@@ -605,31 +618,362 @@ copy_request (gftp_request * req)
 }
 
 
-void
-sig_quit (int signo)
+int
+ptym_open (char *pts_name)
 {
-  signal (signo, sig_quit);
-  siglongjmp (envir, signo == SIGINT ? 1 : 2);
+
+#if defined(SYSV)
+
+  char *tempstr;
+  int fd;
+
+  strcpy (pts_name, "/dev/ptmx");
+  if ((fd = open (pts_name, O_RDWR)) < 0)
+    return (-1);
+
+  if (grantpt (fd) < 0)
+    {
+      close (fd);
+      return (-1);
+    }
+
+  if (unlockpt (fd) < 0)
+    {
+      close (fd);
+      return (-1);
+    }
+
+  if ((tempstr = ptsname (fd)) == NULL)
+    {
+      close (fd);
+      return (-1);
+    }
+
+  strcpy (pts_name, tempstr);
+  return (fd);
+
+#else
+
+  char *pos1, *pos2;
+  int fd;
+
+  strcpy (pts_name, "/dev/ptyXY");
+  for (pos1 = "pqrstuvwxyzPQRST"; *pos1 != '\0'; pos1++) 
+    {
+      pts_name[8] = *pos1;
+      for (pos2 = "0123456789abcdef"; *pos2 != '\0'; pos2++)
+        {
+          pts_name[9] = *pos2;
+          if ((fd = open (pts_name, O_RDWR)) < 0)
+            {
+              if (errno == ENOENT)
+                return (-1);
+              else
+                continue;
+            }
+          pts_name[5] = 't';
+          return (fd);
+        }
+    }
+  return (-1);
+
+#endif
+
+}
+
+
+int
+ptys_open (int fdm, char *pts_name)
+{
+  int fds;
+
+#if !defined(SYSV)
+
+  chmod (pts_name, S_IRUSR | S_IWUSR);
+  chown (pts_name, getuid (), -1);
+
+#endif
+
+  if ((fds = open (pts_name, O_RDWR)) < 0)
+    {
+      close (fdm);
+      return (-1);
+    }
+
+#if defined(SYSV)
+
+  if (ioctl (fds, I_PUSH, "ptem") < 0)
+    {
+      close (fdm);
+      close (fds);
+      return (-1);
+    }
+
+  if (ioctl (fds, I_PUSH, "ldterm") < 0)
+    {
+      close (fdm);
+      close (fds);
+      return (-1);
+    }
+
+  if (ioctl (fds, I_PUSH, "ttcompat") < 0)
+    {
+      close (fdm);
+      close (fds);
+      return (-1);
+    }
+
+#endif
+
+#if !defined(SYSV) && defined(TIOCSCTTY) && !defined(CIBAUD)
+
+  if (ioctl (fds, TIOCSCTTY, (char *) 0) < 0)
+    {
+      close (fdm);
+      return (-1);
+    }
+
+#endif
+
+  return (fds);
+}
+
+
+int
+tty_raw (int fd)
+{
+  struct termios buf;
+
+  if (tcgetattr (fd, &buf) < 0)
+    return (-1);
+
+  buf.c_iflag |= IGNPAR;
+  buf.c_iflag &= ~(ICRNL | ISTRIP | IXON | IGNCR | IXANY | IXOFF | INLCR);
+  buf.c_lflag &= ~(ECHO | ICANON | ISIG | ECHOE | ECHOK | ECHONL);
+#ifdef IEXTEN
+  buf.c_lflag &= ~(IEXTEN);
+#endif
+
+  buf.c_oflag &= ~(OPOST);
+  buf.c_cc[VMIN] = 1;
+  buf.c_cc[VTIME] = 0;
+
+  if (tcsetattr (fd, TCSADRAIN, &buf) < 0)
+    return (-1);
+  return (0);
+}
+
+
+/* We have the caller send us a pointer to a string so we can write the port
+   into it. It makes it easier so we don't have to worry about freeing it 
+   later on, the caller can just send us an auto variable, The string
+   should be at least 6 chars. I know this is messy... */
+
+char **
+make_ssh_exec_args (gftp_request * request, char *execname, 
+                    int use_sftp_subsys, char *portstring)
+{
+  char **args, *oldstr, *tempstr;
+  int i, j;
+
+  args = g_malloc (sizeof (char *) * (num_ssh_extra_params + 10));
+
+  args[0] = ssh_prog_name != NULL && *ssh_prog_name != '\0' ? 
+            ssh_prog_name : "ssh";
+  i = 1;
+  tempstr = g_strconcat (args[0], NULL);
+
+  if (ssh_extra_params_list != NULL)
+    {
+      for (j=0; ssh_extra_params_list[j] != NULL; j++)
+        {
+          oldstr = tempstr;
+          args[i++] = ssh_extra_params_list[j];
+          tempstr = g_strconcat (oldstr, " ", ssh_extra_params_list[j], NULL);
+          g_free (oldstr);
+        }
+    }
+
+  oldstr = tempstr;
+  tempstr = g_strconcat (oldstr, " -e none", NULL);
+  g_free (oldstr);
+  args[i++] = "-e";
+  args[i++] = "none";
+
+  if (request->username && *request->username != '\0')
+    {
+      oldstr = tempstr;
+      tempstr = g_strconcat (oldstr, " -l ", request->username, NULL);
+      g_free (oldstr);
+      args[i++] = "-l";
+      args[i++] = request->username;
+    }
+
+  if (request->port != 0)
+    {
+      g_snprintf (portstring, 6, "%d", request->port);
+      oldstr = tempstr;
+      tempstr = g_strconcat (oldstr, " -p ", portstring, NULL);
+      g_free (oldstr);
+      args[i++] = "-p";
+      args[i++] = portstring;
+    }
+
+  if (use_sftp_subsys)
+    {
+      oldstr = tempstr;
+      tempstr = g_strconcat (oldstr, " ", request->hostname, " -s sftp", NULL);
+      g_free (oldstr);
+      args[i++] = request->hostname;
+      args[i++] = "-s";
+      args[i++] = "sftp";
+      args[i] = NULL;
+    }
+  else
+    {
+      oldstr = tempstr;
+      tempstr = g_strconcat (oldstr, " ", request->hostname, " \"", execname, 
+                             "\"", NULL);
+      g_free (oldstr);
+      args[i++] = request->hostname;
+      args[i++] = execname;
+      args[i] = NULL;
+    }
+
+  request->logging_function (gftp_logging_misc, request->user_data, 
+                             _("Running program %s\n"), tempstr);
+  g_free (tempstr);
+  return (args);
 }
 
 
 char *
-make_temp_filename (char *ext)
+ssh_start_login_sequence (gftp_request * request, int fd)
 {
-  char *destbuf;
- 
-  srand (time (NULL));
-  destbuf = NULL;
-  do
-    {
-      if (destbuf)
-        g_free (destbuf);
-      destbuf = g_strdup_printf ("%s/temp%ld%s", tmp_directory,
-                         1 + (long) (99999999.0 * rand () / (RAND_MAX + 1.0)),
-                         ext == NULL ? "" : ext);
-    }
-  while (access (destbuf, F_OK) != -1);
+  size_t rem, len, diff, lastdiff;
+  int flags, wrotepw, ok;
+  struct timeval tv;
+  char *tempstr;
+  fd_set rdfds;
+  ssize_t rd;
 
-  return (destbuf);
+  rem = len = 100;
+  tempstr = g_malloc0 (len);
+  diff = lastdiff = 0;
+  wrotepw = 0;
+  ok = 1;
+
+  if ((flags = fcntl (fd, F_GETFL, 0)) < 0) 
+    {
+      g_free (tempstr);
+      return (NULL);
+    }
+
+  if (fcntl (fd, F_SETFL, flags | O_NONBLOCK) < 0)
+    {
+      g_free (tempstr);
+      return (NULL);
+    }
+
+  while (1)
+    {
+      FD_ZERO (&rdfds);
+      FD_SET (fd, &rdfds);
+      tv.tv_sec = 5;
+      tv.tv_usec = 0;
+      if (select (fd + 1, &rdfds, NULL, NULL, &tv) < 0)
+        {
+          if (errno == EINTR)
+            continue;
+          ok = 0;
+          break;
+        }
+
+      if ((rd = read (fd, tempstr + diff, rem - 1)) < 0)
+        {
+          if (errno == EINTR)
+            continue;
+          ok = 0;
+          break;
+        }
+      else if (rd == 0)
+        { 
+          ok = 0;
+          break;
+        }
+      tempstr[diff + rd] = '\0';
+      rem -= rd;
+      diff += rd;
+      if (rem <= 1)
+        {
+          tempstr = g_realloc (tempstr, len + 100);
+          tempstr[diff] = '\0';
+          request->logging_function (gftp_logging_recv, request->user_data,
+                                     "%s", tempstr + lastdiff);
+          lastdiff = diff;
+          len += 100;
+          rem = 100;
+        }
+
+      if (!wrotepw && 
+          strlen (tempstr) > 11 && strcmp (tempstr + strlen (tempstr) - 10, 
+                                           "password: ") == 0)
+        {
+          wrotepw = 1;
+          write (fd, request->password, strlen (request->password));
+          write (fd, "\n", 1);
+        }
+
+      else if (!wrotepw && 
+               (strstr (tempstr, "Enter passphrase for RSA key") != NULL ||
+               strstr (tempstr, "Enter passphrase for key '") != NULL))
+        {
+          wrotepw = 1;
+          write (fd, request->password, strlen (request->password));
+          write (fd, "\n", 1);
+        }
+      else if (strlen (tempstr) >= 5 && 
+               strcmp (tempstr + strlen (tempstr) - 5, "xsftp") == 0)
+        break;
+    }
+
+  tempstr[diff] = '\0';
+  request->logging_function (gftp_logging_recv, request->user_data,
+                             "%s\n", tempstr + lastdiff);
+
+  if (ok && fcntl (fd, F_SETFL, flags) < 0)
+    ok = 0;
+
+  if (!ok)
+    {
+      g_free (tempstr);
+      return (NULL);
+    }
+ 
+  return (tempstr);
 }
 
+
+#ifdef G_HAVE_GINT64
+
+gint64
+hton64 (gint64 val)
+{
+  gint64 num;
+  char *pos;
+
+  num = 0;
+  pos = (char *) &num;
+  pos[0] = (val >> 56) & 0xff;
+  pos[1] = (val >> 48) & 0xff;
+  pos[2] = (val >> 40) & 0xff;
+  pos[3] = (val >> 32) & 0xff;
+  pos[4] = (val >> 24) & 0xff;
+  pos[5] = (val >> 16) & 0xff;
+  pos[6] = (val >> 8) & 0xff;
+  pos[7] = val & 0xff;
+  return (num);
+}
+
+#endif
+
