@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  protocols.c - Skeleton functions for the protocols gftp supports         */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -49,7 +49,7 @@ gftp_request_new (void)
   request->sockfd = NULL;
   request->datafd = NULL;
   request->data_type = GFTP_TYPE_BINARY;
-  request->transfer_type = passive_transfer ? gftp_transfer_passive : gftp_transfer_active;
+  gftp_set_config_options (request);
   return (request);
 }
 
@@ -94,6 +94,9 @@ gftp_request_destroy (gftp_request * request)
     g_free (request->last_ftp_response);
   if (request->protocol_data)
     g_free (request->protocol_data);
+  if (request->sftpserv_path)
+    g_free (request->sftpserv_path);
+  memset (request, 0, sizeof (*request));
   g_free (request);
 }
 
@@ -124,6 +127,22 @@ gftp_connect (gftp_request * request)
 
   if (request->connect == NULL)
     return (-2);
+
+  gftp_set_config_options (request);
+
+  if (request->sftpserv_path == NULL)
+    {
+      switch (request->protonum)
+        {
+          case GFTP_SSH_NUM:
+            request->sftpserv_path = g_strconcat (ssh1_sftp_path, NULL);
+            break;
+          case GFTP_SSHV2_NUM:
+            request->sftpserv_path = g_strconcat (ssh2_sftp_path, NULL);
+            break;
+        }
+    }
+
   return (request->connect (request));
 }
 
@@ -133,12 +152,18 @@ gftp_disconnect (gftp_request * request)
 {
   g_return_if_fail (request != NULL);
 
-#ifdef HAVE_GETADDRINFO
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
   if (request->hostp)
     freeaddrinfo (request->hostp);
 #endif
   request->hostp = NULL;
 
+  if (request->sftpserv_path != NULL)
+    {
+      g_free (request->sftpserv_path);
+      request->sftpserv_path = NULL;
+    }
+
   request->cached = 0;
   if (request->disconnect == NULL)
     return;
@@ -146,37 +171,39 @@ gftp_disconnect (gftp_request * request)
 }
 
 
-long
-gftp_get_file (gftp_request * request, const char *filename, long startsize)
+size_t
+gftp_get_file (gftp_request * request, const char *filename, FILE * fd,
+               size_t startsize)
 {
   g_return_val_if_fail (request != NULL, -2);
 
   request->cached = 0;
   if (request->get_file == NULL)
     return (-2);
-  return (request->get_file (request, filename, startsize));
+  return (request->get_file (request, filename, fd, startsize));
 }
 
 
 int
-gftp_put_file (gftp_request * request, const char *filename, long startsize,
-               long totalsize)
+gftp_put_file (gftp_request * request, const char *filename, FILE * fd,
+               size_t startsize, size_t totalsize)
 {
   g_return_val_if_fail (request != NULL, -2);
 
   request->cached = 0;
   if (request->put_file == NULL)
     return (-2);
-  return (request->put_file (request, filename, startsize, totalsize));
+  return (request->put_file (request, filename, fd, startsize, totalsize));
 }
 
 
 long
-gftp_transfer_file (gftp_request * fromreq, const char *fromfile,
-		    long fromsize, gftp_request * toreq, const char *tofile,
-		    long tosize)
+gftp_transfer_file (gftp_request * fromreq, const char *fromfile, 
+                    FILE * fromfd, size_t fromsize, 
+                    gftp_request * toreq, const char *tofile,
+                    FILE * tofd, size_t tosize)
 {
-  long size;
+  size_t size;
 
   g_return_val_if_fail (fromreq != NULL, -2);
   g_return_val_if_fail (fromfile != NULL, -2);
@@ -193,10 +220,10 @@ gftp_transfer_file (gftp_request * fromreq, const char *fromfile,
 
   fromreq->cached = 0;
   toreq->cached = 0;
-  if ((size = gftp_get_file (fromreq, fromfile, tosize)) < 0)
+  if ((size = gftp_get_file (fromreq, fromfile, fromfd, tosize)) < 0)
     return (-2);
 
-  if (gftp_put_file (toreq, tofile, tosize, size) != 0)
+  if (gftp_put_file (toreq, tofile, tofd, tosize, size) != 0)
     {
       gftp_end_transfer (fromreq);
       return (-2);
@@ -221,7 +248,7 @@ gftp_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 
   FD_ZERO (&fset);
   FD_SET (fileno (request->datafd), &fset);
-  tv.tv_sec = read_timeout;
+  tv.tv_sec = request->read_timeout;
   tv.tv_usec = 0;
   if (select (fileno (request->datafd) + 1, &fset, NULL, NULL, &tv) <= 0)
     {
@@ -265,7 +292,7 @@ gftp_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
     {
       FD_ZERO (&fset);
       FD_SET (fileno (request->datafd), &fset);
-      tv.tv_sec = read_timeout;
+      tv.tv_sec = request->read_timeout;
       tv.tv_usec = 0;
       if (!select (fileno (request->datafd) + 1, NULL, &fset, NULL, &tv))
         {
@@ -404,13 +431,15 @@ gftp_parse_url (gftp_request * request, const char *url)
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (url != NULL, -2);
 
-  if ((pos = strstr (url, "://")) != NULL)
+  for (stpos = url; *stpos == ' '; stpos++);
+
+  if ((pos = strstr (stpos, "://")) != NULL)
     {
       *pos = '\0';
 
       for (i = 0; gftp_protocols[i].url_prefix; i++)
         {
-          if (strcmp (gftp_protocols[i].url_prefix, url) == 0)
+          if (strcmp (gftp_protocols[i].url_prefix, stpos) == 0)
             break;
         }
 
@@ -419,18 +448,27 @@ gftp_parse_url (gftp_request * request, const char *url)
       *pos = ':';
     }
   else
-    i = GFTP_FTP_NUM;
+    {
+      for (i = 0; gftp_protocols[i].url_prefix; i++)
+        {
+          if (strcmp (gftp_protocols[i].name, default_protocol) == 0)
+            break;
+        }
+
+      if (gftp_protocols[i].url_prefix == NULL)
+        i = GFTP_FTP_NUM;
+    }
 
   gftp_protocols[i].init (request);
 
   if (i == GFTP_LOCAL_NUM)
     {
-      request->directory = g_malloc (strlen (url + 6) + 1);
-      strcpy (request->directory, url + 6);
+      request->directory = g_malloc (strlen (stpos + 6) + 1);
+      strcpy (request->directory, stpos + 6);
       return (0);
     }
 
-  for (stpos = url; *stpos == ' '; stpos++);
+  for (; *stpos == ' '; stpos++);
   str = g_malloc (strlen (stpos) + 1);
   strcpy (str, stpos);
   for (pos = str + strlen (str) - 1; *pos == ' '; pos--)
@@ -470,9 +508,11 @@ gftp_parse_url (gftp_request * request, const char *url)
     {
       gftp_set_username (request, "anonymous");
       if (request->password)
-	g_free (request->password);
+        {
+	  g_free (request->password);
+          request->password = NULL;
+        }
       gftp_set_password (request, emailaddr);
-      request->password = NULL;
     }
 
   /* Now get the hostname and an optional port and optional directory */
@@ -781,7 +821,7 @@ gftp_need_proxy (gftp_request * request, char *service)
   GList * templist;
   gint32 netaddr;
   char *pos;
-#ifdef HAVE_GETADDRINFO
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
   struct addrinfo hints;
   int port, errnum;
   char serv[8];
@@ -1006,6 +1046,7 @@ parse_time (char **str)
 
   startpos = *str;
   memset (&tt, 0, sizeof (tt));
+  tt.tm_isdst = -1;
   if (isdigit ((int) startpos[0]) && startpos[2] == '-')
     {
       /* This is how DOS will return the date/time */
@@ -1427,8 +1468,7 @@ gftp_get_all_subdirs (gftp_transfer * transfer,
       if (transfer->toreq)
         curfle->destfile = g_strconcat (transfer->toreq->directory, "/", 
                                         curfle->file, NULL);
-      newname = g_strconcat (transfer->fromreq->directory, "/", curfle->file, 
-                             NULL);
+      newname = g_strconcat (transfer->fromreq->directory, transfer->fromreq->directory[strlen (transfer->fromreq->directory) - 1] == '/' ? "" : "/", curfle->file, NULL);
       g_free (curfle->file);
       curfle->file = newname;
 
@@ -1531,6 +1571,7 @@ gftp_gen_dir_hash (gftp_request * request)
           gftp_file_destroy (fle);
         }
       gftp_end_transfer (request);
+      g_free (fle);
     }
   else
     {
@@ -1589,7 +1630,7 @@ gftp_get_file_transfer_mode (char *filename, int def)
 }
 
 
-#ifdef HAVE_GETADDRINFO
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
 int
 get_port (struct addrinfo *addr)
 {
@@ -1614,7 +1655,7 @@ gftp_connect_server (gftp_request * request, char *service)
 {
   char *connect_host, *disphost;
   int port, sock;
-#ifdef HAVE_GETADDRINFO
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
   struct addrinfo hints, *res;
   char serv[8];
   int errnum;
@@ -1780,6 +1821,40 @@ gftp_connect_server (gftp_request * request, char *service)
 }
 
 
+void
+gftp_set_config_options (gftp_request * request)
+{
+  request->transfer_type = passive_transfer ? gftp_transfer_passive : gftp_transfer_active;
+  request->connect_timeout = connect_timeout;
+  request->read_timeout = read_timeout;
+  request->retries = retries;
+  request->sleep_time = sleep_time;
+  request->maxkbs = maxkbs;
+
+  if (request->protonum == GFTP_SSHV2_NUM || 
+      request->protonum == GFTP_SSH_NUM)
+    request->need_userpass = ssh_need_userpass;
+}
+
+
+void
+gftp_set_sftpserv_path (gftp_request * request, char *path)
+{
+  g_return_if_fail (request != NULL);
+
+  if (request->sftpserv_path)
+    g_free (request->sftpserv_path);
+
+  if (path != NULL && *path != '\0')
+    {
+      request->sftpserv_path = g_malloc (strlen (path) + 1);
+      strcpy (request->sftpserv_path, path);
+    }
+  else
+    request->sftpserv_path = NULL;
+}
+
+
 void
 print_file_list (GList * list)
 {
