@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  rfc2068.c - General purpose routines for the HTTP protocol               */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -21,15 +21,16 @@
 
 static int rfc2068_connect 			( gftp_request * request );
 static void rfc2068_disconnect 			( gftp_request * request );
-static long rfc2068_get_file 			( gftp_request * request, 
+static off_t rfc2068_get_file 			( gftp_request * request, 
 						  const char *filename, 
-						  long startsize );
+						  FILE * fd,
+						  off_t startsize );
 static size_t rfc2068_get_next_file_chunk 	( gftp_request * request, 
 						  char *buf, 
 						  size_t size );
 static int rfc2068_end_transfer 		( gftp_request * request );
 static int rfc2068_list_files 			( gftp_request * request );
-static long rfc2068_get_file_size 		( gftp_request * request, 
+static off_t rfc2068_get_file_size 		( gftp_request * request, 
 						  const char *filename );
 static int rfc2068_get_next_file 		( gftp_request * request, 
 						  gftp_file * fle, 
@@ -139,17 +140,21 @@ rfc2068_disconnect (gftp_request * request)
 }
 
 
-static long
-rfc2068_get_file (gftp_request * request, const char *filename, long startsize)
+static off_t  
+rfc2068_get_file (gftp_request * request, const char *filename, FILE * fd,
+                  off_t startsize)
 {
   char *tempstr, *extrahdr, *pos;
-  unsigned long size;
+  off_t size;
   int restarted;
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, -2);
   g_return_val_if_fail (filename != NULL, -2);
 
+  if (fd != NULL)
+    request->datafd = fd;
+
   if (request->datafd == NULL && rfc2068_connect (request) != 0)
     return (-2);
 
@@ -177,6 +182,7 @@ rfc2068_get_file (gftp_request * request, const char *filename, long startsize)
     extrahdr = NULL;
   else
     {
+      /* FIXME - IRIX complains */
       extrahdr = g_strdup_printf ("Range: bytes=%ld-\n", startsize);
       request->logging_function (gftp_logging_misc, request->user_data,
 			       _("Starting the file transfer at offset %ld\n"), 
@@ -223,7 +229,7 @@ rfc2068_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 
   FD_ZERO (&fset);
   FD_SET (fileno (request->datafd), &fset);
-  tv.tv_sec = read_timeout;
+  tv.tv_sec = request->read_timeout;
   tv.tv_usec = 0;
   if (select (fileno (request->datafd) + 1, &fset, NULL, NULL, &tv) <= 0)
     {
@@ -325,7 +331,7 @@ rfc2068_list_files (gftp_request * request)
 }
 
 
-static long
+static off_t 
 rfc2068_get_file_size (gftp_request * request, const char *filename)
 {
   char *tempstr, *pos;
@@ -372,6 +378,7 @@ rfc2068_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
 {
   rfc2068_params * params;
   char tempstr[255];
+  size_t len;
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, -2);
@@ -419,10 +426,15 @@ rfc2068_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
       gftp_file_destroy (fle);
       return (-2);
     }
-  request->last_dir_entry = g_malloc (strlen (tempstr) + 1);
-  strcpy (request->last_dir_entry, tempstr);
-  request->last_dir_entry_len = strlen (request->last_dir_entry);
-  return (feof (fd) ? 0 : request->last_dir_entry_len);
+
+  len = strlen (tempstr);
+  if (!request->cached)
+    {
+      request->last_dir_entry = g_malloc (len + 1);
+      strcpy (request->last_dir_entry, tempstr);
+      request->last_dir_entry_len = len;
+    }
+  return (feof (fd) ? 0 : len);
 }
 
 
@@ -714,7 +726,7 @@ base64_encode (char *str)
 /* The standard to Base64 encoding can be found in RFC2045 */
 
   char *newstr, *newpos, *fillpos, *pos;
-  unsigned char table[63], encode[3];
+  unsigned char table[64], encode[3];
   int i, num;
 
   for (i = 0; i < 26; i++)
