@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  rfc959.c - General purpose routines for the FTP protocol (RFC 959)       */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -21,24 +21,26 @@
 
 static int rfc959_connect 			( gftp_request * request );
 static void rfc959_disconnect 			( gftp_request * request );
-static long rfc959_get_file 			( gftp_request * request, 
+static off_t rfc959_get_file 			( gftp_request * request, 
 						  const char *filename,
-						  long startsize );
+						  FILE * fd,
+						  off_t startsize );
 static int rfc959_put_file 			( gftp_request * request, 
 						  const char *filename,
-						  long startsize,
-						  long totalsize );
-static long rfc959_transfer_file 		( gftp_request *fromreq, 
+						  FILE * fd,
+						  off_t startsize,
+						  off_t totalsize );
+static off_t rfc959_transfer_file 		( gftp_request *fromreq, 
 						  const char *fromfile, 
-						  long fromsize, 
+						  off_t fromsize, 
 						  gftp_request *toreq, 
 						  const char *tofile, 
-						  long tosize );
+						  off_t tosize );
 static int rfc959_end_transfer 			( gftp_request * request );
 static int rfc959_list_files 			( gftp_request * request );
 static int rfc959_set_data_type 		( gftp_request * request, 
 						  int data_type );
-static long rfc959_get_file_size 		( gftp_request * request, 
+static off_t rfc959_get_file_size 		( gftp_request * request, 
 						  const char *filename );
 static int rfc959_data_connection_new 		( gftp_request * request );
 static int rfc959_accept_active_connection 	( gftp_request * request );
@@ -274,8 +276,9 @@ rfc959_disconnect (gftp_request * request)
 }
 
 
-static long
-rfc959_get_file (gftp_request * request, const char *filename, long startsize)
+static off_t  
+rfc959_get_file (gftp_request * request, const char *filename, FILE * fd,
+                 off_t startsize)
 {
   char *command, *tempstr, resp;
   int ret, flags;
@@ -285,6 +288,9 @@ rfc959_get_file (gftp_request * request, const char *filename, long startsize)
   g_return_val_if_fail (filename != NULL, -2);
   g_return_val_if_fail (request->sockfd != NULL, -2);
 
+  if (fd != NULL)
+    request->datafd = fd;
+
   if (request->datafd == NULL && (ret = rfc959_data_connection_new (request)) < 0)
     return (ret);
 
@@ -294,7 +300,8 @@ rfc959_get_file (gftp_request * request, const char *filename, long startsize)
 
   if (startsize > 0)
     {
-      command = g_strdup_printf ("REST %ld\r\n", startsize);
+      /* FIXME - IRIX complains */
+      command = g_strdup_printf ("REST %ld\r\n", startsize); 
       resp = rfc959_send_command (request, command);
       g_free (command);
 
@@ -327,8 +334,8 @@ rfc959_get_file (gftp_request * request, const char *filename, long startsize)
 
 
 static int
-rfc959_put_file (gftp_request * request, const char *filename, long startsize,
-                 long totalsize)
+rfc959_put_file (gftp_request * request, const char *filename, FILE * fd,
+                 off_t startsize, off_t totalsize)
 {
   char *command, *tempstr, resp;
   int ret, flags;
@@ -338,6 +345,9 @@ rfc959_put_file (gftp_request * request, const char *filename, long startsize,
   g_return_val_if_fail (filename != NULL, -2);
   g_return_val_if_fail (request->sockfd != NULL, -2);
 
+  if (fd != NULL)
+    fd = request->datafd;
+
   if (request->datafd == NULL && (ret = rfc959_data_connection_new (request)) < 0)
     return (ret);
 
@@ -347,7 +357,8 @@ rfc959_put_file (gftp_request * request, const char *filename, long startsize,
 
   if (startsize > 0)
     {
-      command = g_strdup_printf ("REST %ld\r\n", startsize);
+      /* FIXME - IRIX complains */
+      command = g_strdup_printf ("REST %ld\r\n", startsize);  
       resp = rfc959_send_command (request, command);
       g_free (command);
       if (resp != '3')
@@ -366,10 +377,10 @@ rfc959_put_file (gftp_request * request, const char *filename, long startsize,
   return (0);
 }
 
-static long
+static off_t  
 rfc959_transfer_file (gftp_request *fromreq, const char *fromfile, 
-                      long fromsize, gftp_request *toreq, const char *tofile, 
-                      long tosize)
+                      off_t fromsize, gftp_request *toreq, 
+                      const char *tofile, off_t tosize)
 {
   char *tempstr, *pos, *endpos;
 
@@ -476,6 +487,7 @@ int
 rfc959_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
 {
   char tempstr[255];
+  size_t len;
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_FTP_NUM, -2);
@@ -513,10 +525,14 @@ rfc959_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
     }
   while (1);
 
-  request->last_dir_entry = g_malloc (strlen (tempstr) + 1);
-  strcpy (request->last_dir_entry, tempstr);
-  request->last_dir_entry_len = strlen (request->last_dir_entry); 
-  return (request->last_dir_entry_len);
+  len = strlen (tempstr);
+  if (!request->cached)
+    {
+      request->last_dir_entry = g_malloc (len + 1);
+      strcpy (request->last_dir_entry, tempstr);
+      request->last_dir_entry_len = len;
+    }
+  return (len);
 }
 
 
@@ -543,7 +559,7 @@ rfc959_set_data_type (gftp_request * request, int data_type)
 }
 
 
-static long
+static off_t
 rfc959_get_file_size (gftp_request * request, const char *filename)
 {
   char *tempstr;
@@ -612,7 +628,7 @@ rfc959_data_connection_new (gftp_request * request)
           close (sock);
           return (-2);
         }
-      if (sscanf (pos, "%d,%d,%d,%d,%d,%d", &temp[0], &temp[1], &temp[2],
+      if (sscanf (pos, "%u,%u,%u,%u,%u,%u", &temp[0], &temp[1], &temp[2],
                   &temp[3], &temp[4], &temp[5]) != 6)
         {
           rfc959_disconnect (request);
@@ -636,6 +652,7 @@ rfc959_data_connection_new (gftp_request * request)
     }
   else
     {
+      /* FIXME - 3rd arg */
       getsockname (fileno (request->sockfd), (struct sockaddr *) &data_addr,
 		   &data_addr_len);
       data_addr.sin_port = 0;
@@ -648,6 +665,8 @@ rfc959_data_connection_new (gftp_request * request)
 	  close (sock);
 	  return (-1);
 	}
+
+      /* FIXME - 3rd arg */
       getsockname (sock, (struct sockaddr *) &data_addr, &data_addr_len);
       if (listen (sock, 1) == -1)
 	{
@@ -661,7 +680,7 @@ rfc959_data_connection_new (gftp_request * request)
 	}
       pos = (char *) &data_addr.sin_addr;
       pos1 = (char *) &data_addr.sin_port;
-      command = g_strdup_printf ("PORT %d,%d,%d,%d,%d,%d\r\n",
+      command = g_strdup_printf ("PORT %u,%u,%u,%u,%u,%u\r\n",
 				 pos[0] & 0xff, pos[1] & 0xff, pos[2] & 0xff,
 				 pos[3] & 0xff, pos1[0] & 0xff,
 				 pos1[1] & 0xff);
@@ -701,6 +720,7 @@ rfc959_accept_active_connection (gftp_request * request)
   g_return_val_if_fail (request->transfer_type == gftp_transfer_active, -2);
 
   cli_addr_len = sizeof (cli_addr);
+  /* FIXME - 3rd arg */
   if ((infd = accept (fileno (request->datafd), (struct sockaddr *) &cli_addr,
        &cli_addr_len)) == -1)
     {
@@ -744,24 +764,36 @@ rfc959_send_command (gftp_request * request, const char *command)
 static int
 write_to_socket (gftp_request *request, const char *command)
 {
-  int ret;
+  ssize_t numleft, wrote;
+  const char *pos;
 
-  do
+  pos = command;
+  numleft = strlen (command);
+  while (numleft > 0)
     {
-      fwrite (command, 1, strlen (command), request->sockfd_write);
-      ret = ferror (request->sockfd);
-      if (ret != 0 && errno != EINTR)
+      wrote = fwrite (pos, 1, numleft, request->sockfd_write);
+      if (ferror (request->sockfd_write))
         {
-          request->logging_function (gftp_logging_send, request->user_data,
-                                     "Error writing %s to socket: %s\n",
-                                     command, g_strerror (errno));
+          request->logging_function (gftp_logging_error, request->user_data,
+                                   _("Error: Could not write to socket: %s\n"),
+                                   g_strerror (errno));
           rfc959_disconnect (request);
           return (-1);
         }
-      else
-        fflush (request->sockfd_write);
+      numleft -= wrote;
+      pos += wrote;
     }
-  while (ret != 0 && errno == EINTR);
+
+  fflush (request->sockfd_write);
+  if (ferror (request->sockfd_write))
+    {
+      request->logging_function (gftp_logging_error, request->user_data,
+                                 _("Error: Could not write to socket: %s\n"),
+                                 g_strerror (errno));
+      rfc959_disconnect (request);
+      return (-1);
+    }
+
   return (0);
 }
     
@@ -1000,9 +1032,9 @@ parse_ftp_proxy_string (gftp_request * request)
   while (*endpos != '\0')
     {
       tempport = NULL;
-      if (*endpos == '%' && tolower (*(endpos + 1)) == 'p')
+      if (*endpos == '%' && tolower ((int) *(endpos + 1)) == 'p')
 	{
-	  switch (tolower (*(endpos + 2)))
+	  switch (tolower ((int) *(endpos + 2)))
 	    {
 	    case 'u':
 	      newval = request->proxy_username;
@@ -1025,9 +1057,9 @@ parse_ftp_proxy_string (gftp_request * request)
 	      continue;
 	    }
 	}
-      else if (*endpos == '%' && tolower (*(endpos + 1)) == 'h')
+      else if (*endpos == '%' && tolower ((int) *(endpos + 1)) == 'h')
 	{
-	  switch (tolower (*(endpos + 2)))
+	  switch (tolower ((int) *(endpos + 2)))
 	    {
 	    case 'u':
 	      newval = request->username;
@@ -1050,7 +1082,7 @@ parse_ftp_proxy_string (gftp_request * request)
 	      continue;
 	    }
 	}
-      else if (*endpos == '%' && tolower (*(endpos + 1)) == 'n')
+      else if (*endpos == '%' && tolower ((int) *(endpos + 1)) == 'n')
 	{
 	  *endpos = '\0';
 	  oldstr = newstr;
