@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  ssh.c - functions that will use the ssh protocol                         */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -52,6 +52,7 @@ typedef struct ssh_parms_tag
     char channel;		/* Data channel we are writing to */
 } ssh_parms;
 
+
 typedef struct ssh_message_tag
 {
   char channel,
@@ -60,18 +61,19 @@ typedef struct ssh_message_tag
   void *data;
 } ssh_message;
 
+
 static void ssh_destroy 		( gftp_request * request );
 static int ssh_connect 			( gftp_request * request );
-static char *start_login_sequence 	( gftp_request * request,
-					  int fd );
 static void ssh_disconnect 		( gftp_request * request );
-static long ssh_get_file 		( gftp_request * request, 
+static off_t ssh_get_file 		( gftp_request * request, 
 					  const char *filename,
-					  long startsize );
+					  FILE * fd,
+					  off_t startsize );
 static int ssh_put_file 		( gftp_request * request, 
 					  const char *filename,
-					  long startsize,
-					  long totalsize );
+					  FILE * fd,
+					  off_t startsize,
+					  off_t totalsize );
 static size_t ssh_get_next_file_chunk 	( gftp_request * request, 
 					  char *buf, 
 					  size_t size );
@@ -118,10 +120,6 @@ static void ssh_log_command 		( gftp_request * request,
 					  const char *command, 
 					  size_t len,
 					  int direction );
-static int ptym_open 			( char *pts_name );
-static int ptys_open 			( int fdm, 
-					  char *pts_name );
-static int tty_raw 			( int fd );
 static size_t ssh_remove_spaces 	( char *string );
 
 void
@@ -155,7 +153,7 @@ ssh_init (gftp_request * request)
   request->url_prefix = "ssh";
   request->protocol_name = "SSH";
   request->need_hostport = 1;
-  request->need_userpass = 1;
+  request->need_userpass = ssh_need_userpass;
   request->use_cache = 1;
   request->use_threads = 1;
   request->always_connected = 0;
@@ -183,9 +181,10 @@ ssh_destroy (gftp_request * request)
 static int
 ssh_connect (gftp_request * request)
 {
-  char *args[10], *tempstr, pts_name[20], port[6];
-  int i, version, fdm, fds;
+  char **args, *tempstr, pts_name[20], *p1, p2, *exepath, port[6];
+  int version, fdm, fds, s[2];
   ssh_message message;
+  const gchar *errstr;
   ssh_parms *params;
   pid_t child;
 
@@ -201,63 +200,100 @@ ssh_connect (gftp_request * request)
 			     _("Opening SSH connection to %s\n"),
                              request->hostname);
 
-  args[0] = "ssh";
-  i = 1;
-  if (request->username)
+  if (request->sftpserv_path == NULL ||
+      *request->sftpserv_path == '\0') 
     {
-      args[i++] = "-l";
-      args[i++] = request->username;
+      p1 = "";
+      p2 = ' ';
     }
-  if (request->port != 0)
+  else
     {
-      args[i++] = "-p";
-      g_snprintf (port, sizeof (port), "%d", request->port);
-      args[i++] = port;
+      p1 = request->sftpserv_path;
+      p2 = '/';
     }
-  args[i++] = request->hostname;
-  args[i++] = "sftpserv";
-  args[i] = NULL;
 
-  if ((fdm = ptym_open (pts_name)) < 0)
+  *port = '\0';
+  exepath = g_strdup_printf ("%s%csftpserv", p1, p2);
+  args = make_ssh_exec_args (request, exepath, 0, port);
+
+   if (ssh_use_askpass)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Cannot open master pty %s\n"), pts_name);
-      return (-2);
+      fdm = fds = 0;
+      if (socketpair (AF_LOCAL, SOCK_STREAM, 0, s) < 0)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                     _("Cannot create a socket pair: %s\n"),
+                                     g_strerror (errno));
+          return (-2);
+        }
     }
- 
+  else
+    {
+      s[0] = s[1] = 0;
+      if ((fdm = ptym_open (pts_name)) < 0)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                _("Cannot open master pty %s: %s\n"), pts_name,
+                                g_strerror (errno));
+          return (-2);
+        }
+    }
+
   if ((child = fork ()) == 0)
     {
       setsid ();
-      if ((fds = ptys_open (fdm, pts_name)) < 0)
-        return (-1);
-      close (fdm);
+      if (ssh_use_askpass)
+        {
+          close (s[0]);
+          fds = s[1];
+        }
+      else
+        {
+          if ((fds = ptys_open (fdm, pts_name)) < 0)
+            {
+              printf ("Cannot open slave pts %s: %s\n", pts_name,
+                      g_strerror (errno));
+              return (-1);
+            }
+          close (fdm);
+        }
+
       dup2 (fds, 0);
       dup2 (fds, 1);
       dup2 (fds, 2);
-      if (fds > 2)
+      if (!ssh_use_askpass && fds > 2)
         close (fds);
-      execvp ("ssh", args);  
+      execvp (ssh_prog_name != NULL && *ssh_prog_name != '\0' ?
+              ssh_prog_name : "ssh", args);  
 
       tempstr = _("Error: Cannot execute ssh: ");
       write (1, tempstr, strlen (tempstr));
-      tempstr = g_strerror (errno);
-      write (1, tempstr, strlen (tempstr));
+      errstr = g_strerror (errno);
+      write (1, errstr, strlen (errstr));
       write (1, "\n", 1);
       return (-1);
     }
   else if (child > 0)
     {
+      if (ssh_use_askpass)
+        {
+          close (s[1]);
+          fdm = s[0];
+        }
       tty_raw (fdm);
-      tempstr = start_login_sequence (request, fdm);
-
+      tempstr = ssh_start_login_sequence (request, fdm);
       if (!tempstr || 
           !(strlen (tempstr) > 4 && strcmp (tempstr + strlen (tempstr) - 5, 
                                             "xsftp") == 0))
         {
           request->logging_function (gftp_logging_error, request->user_data,
 		_("Error: Received wrong init string from server\n"));
+          g_free (args);
+          g_free (exepath);
           return (-2);
         }
+      g_free (args);
+      g_free (exepath);
       g_free (tempstr);
 
       request->sockfd = request->datafd = fdopen (fdm, "rb");
@@ -280,6 +316,7 @@ ssh_connect (gftp_request * request)
       request->logging_function (gftp_logging_error, request->user_data,
                                  _("Cannot fork another process: %s\n"),
                                  g_strerror (errno));
+      g_free (args);
       return (-1);
     }
 
@@ -308,124 +345,6 @@ ssh_connect (gftp_request * request)
 }
 
 
-static char *
-start_login_sequence (gftp_request * request, int fd)
-{
-  size_t rem, len, diff, lastdiff;
-  int flags, wrotepw, ok;
-  char *tempstr, *pos;
-  fd_set rdfds;
-  ssize_t rd;
-
-  rem = len = 100;
-  tempstr = g_malloc0 (len);
-  diff = lastdiff = 0;
-  wrotepw = 0;
-  ok = 1;
-
-  if ((flags = fcntl (fd, F_GETFL, 0)) < 0) 
-    {
-      g_free (tempstr);
-      return (NULL);
-    }
-
-  if (fcntl (fd, F_SETFL, flags | O_NONBLOCK) < 0)
-    {
-      g_free (tempstr);
-      return (NULL);
-    }
-
-  while (1)
-    {
-      FD_ZERO (&rdfds);
-      FD_SET (fd, &rdfds);
-      if (select (fd + 1, &rdfds, NULL, NULL, NULL) < 0)
-        {
-          if (errno == EINTR)
-            continue;
-          ok = 0;
-          break;
-        }
-
-      if ((rd = read (fd, tempstr + diff, rem)) < 0)
-        {
-          if (errno == EINTR)
-            continue;
-          ok = 0;
-          break;
-        }
-      rem -= rd;
-      diff += rd;
-      if (rem == 0)
-        {
-          tempstr = g_realloc (tempstr, len + 100);
-          tempstr[diff] = '\0';
-          request->logging_function (gftp_logging_recv, request->user_data,
-                                     "%s", tempstr + lastdiff);
-          lastdiff = diff;
-          len += 100;
-          rem = 100;
-        }
-
-      if (strlen (tempstr) > 11 && strcmp (tempstr + strlen (tempstr) - 10, 
-                                           "password: ") == 0)
-        {
-          if (wrotepw)
-            {
-              ok = 0;
-              break;
-            }
-          wrotepw = 1;
-          write (fd, request->password, strlen (request->password));
-          write (fd, "\n", 1);
-        }
-      else if (strlen (tempstr) > 4 && strcmp (tempstr + strlen (tempstr) - 3,
-                                               "': ") == 0)
-        {
-          pos = tempstr + strlen (tempstr) - 4;
-          while (pos > tempstr && *pos != '\'')
-            pos--;
-          if (pos > tempstr && *pos == '\'' 
-              && strncmp ("RSA key", pos - 8, 7) == 0)
-            {
-              if (wrotepw)
-                {
-                  ok = 0;
-                  break;
-                }
-              wrotepw = 1;
-              write (fd, request->password, strlen (request->password));
-              write (fd, "\n", 1);
-            }
-        }
-      else if (strlen (tempstr) >= 5 && 
-               strcmp (tempstr + strlen (tempstr) - 5, "xsftp") == 0)
-        {
-          read (fd, tempstr + diff, 1); /* Read NUL */
-          break;
-        }
-
-      if (rd == 0)
-        break;
-    }
-
-  tempstr[diff] = '\0';
-  request->logging_function (gftp_logging_recv, request->user_data,
-                             "%s\n", tempstr + lastdiff);
-
-  if (ok && fcntl (fd, F_SETFL, flags) < 0)
-    ok = 0;
-
-  if (!ok)
-    {
-      g_free (tempstr);
-      return (NULL);
-    }
- 
-  return (tempstr);
-}
-
-
 static void
 ssh_disconnect (gftp_request * request)
 {
@@ -443,18 +362,19 @@ ssh_disconnect (gftp_request * request)
 }
 
 
-static 
-long
-ssh_get_file (gftp_request * request, const char *filename, long startsize)
+static off_t 
+ssh_get_file (gftp_request * request, const char *filename, FILE * fd,
+              off_t startsize)
 {
   ssh_message message;
   ssh_parms *params;
-  long retsize;
+  off_t retsize;
   int ret;
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_SSH_NUM, -2);
   g_return_val_if_fail (filename != NULL, -2);
+  /* fd ignored for this protocol */
 
   params = request->protocol_data;
   params->channel = 0; 
@@ -500,8 +420,8 @@ ssh_get_file (gftp_request * request, const char *filename, long startsize)
 
 
 static int
-ssh_put_file (gftp_request * request, const char *filename, long startsize,
-              long totalsize)
+ssh_put_file (gftp_request * request, const char *filename, FILE * fd,
+              off_t startsize, off_t totalsize)
 {
   ssh_message message;
   ssh_parms *params;
@@ -510,6 +430,7 @@ ssh_put_file (gftp_request * request, const char *filename, long startsize,
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_SSH_NUM, -2);
   g_return_val_if_fail (filename != NULL, -2);
+  /* fd ignored for this protocol */
 
   params = request->protocol_data;
   params->channel = 0;
@@ -575,7 +496,7 @@ ssh_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
             g_free (message.data);
             return (-1);
           }
-
+        g_free (message.data);
         return (0);
       case -1:
         g_free (message.data);
@@ -608,10 +529,18 @@ ssh_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 static int
 ssh_end_transfer (gftp_request * request)
 {
+  ssh_parms *params;
+
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_SSH_NUM, -2);
   g_return_val_if_fail (request->datafd != NULL, -2);
 
+  params = request->protocol_data;
+  if (params->buffer)
+    g_free (params->buffer);
+  params->buffer = params->pos = NULL;
+  params->enddata = 0;
+
   return (0);
 }
 
@@ -677,26 +606,38 @@ ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
       request->last_dir_entry_len = 0;
     }
 
+  len = 0;
   while (1)
     {
       if (params->enddata && params->buffer == NULL)
         {
-            request->logging_function (gftp_logging_misc, request->user_data,
+          request->logging_function (gftp_logging_misc, request->user_data,
 			         _("Finished retrieving directory listing\n"));
-            params->enddata = 0;
-            request->last_dir_entry_len = 6;
-            request->last_dir_entry = g_malloc0 (request->last_dir_entry_len);
-            request->last_dir_entry[0] = params->channel;
-            request->last_dir_entry[1] = ENDDATA;
-            return (0);
+          params->enddata = 0;
+          return (0);
         }
 
-      ret = DATA;
       if (!params->enddata && 
           (!params->pos || strchr (params->pos, '\n') == NULL)) 
         {
           if ((ret = ssh_read_response (request, &message, fd)) < 0)
-            return (-2);
+            {
+              if (message.data)
+                g_free (message.data);
+              return (-2);
+            }
+
+          if (!request->cached)
+            {
+              request->last_dir_entry_len = message.len + 6;
+              request->last_dir_entry = g_malloc (request->last_dir_entry_len + 1);
+              request->last_dir_entry[0] = message.channel;
+              request->last_dir_entry[1] = message.command;
+              len = htonl (message.len);
+              memcpy (&request->last_dir_entry[2], &len, 4);
+              memcpy (&request->last_dir_entry[6], message.data, message.len);
+            }
+
           if (ret == DATA)
             {
               if (params->pos == NULL || *params->pos == '\0')
@@ -713,10 +654,15 @@ ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
                   strcpy (params->buffer, params->pos);
                   strcat (params->buffer, (char *) message.data);
                   g_free (tempstr);
+                  g_free (message.data);
                   params->pos = params->buffer;
                 } 
             }
+          else
+            g_free (message.data);
         }
+      else
+        ret = DATA;
 
       switch (ret)
         {
@@ -727,12 +673,15 @@ ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
             while (*pos == ' ' || *pos == '\t')
               pos++;
             if (*pos == '\0')
-              break;
+              {
+                g_free (tempstr);
+                break;
+              }
 
             if (gftp_parse_ls (tempstr, fle) != 0)
 	      {
-	        if (strncmp (tempstr, "total", strlen("total")) &&
-		    strncmp (tempstr, _("total"), strlen(_("total"))))
+	        if (strncmp (tempstr, "total", strlen ("total")) &&
+		    strncmp (tempstr, _("total"), strlen (_("total"))))
 	          request->logging_function (gftp_logging_misc, 
                                        request->user_data,
 				       _("Warning: Cannot parse listing %s\n"),
@@ -742,16 +691,8 @@ ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
                 tempstr = NULL;
 	        continue;
               }
-            request->last_dir_entry_len = strlen (tempstr) + 8;
-            request->last_dir_entry = g_malloc (request->last_dir_entry_len);
-            request->last_dir_entry[0] = params->channel;
-            request->last_dir_entry[1] = DATA;
-            len = htonl (strlen (tempstr) + 2);
-            memcpy (&request->last_dir_entry[2], &len, 4);
-            strcpy (&request->last_dir_entry[6], tempstr);
-            len = strlen (tempstr) + 6;
-            request->last_dir_entry[len] = '\n';
-            request->last_dir_entry[len + 1] = '\0';
+            len = strlen (tempstr);
+            g_free (tempstr);
             break;
           case ENDDATA:
             params->enddata = 1;
@@ -763,11 +704,11 @@ ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
             return (-2);
 	}
 
-      if (request->last_dir_entry != NULL)
+      if (ret == DATA)
         break;
     }
 
-  return (request->last_dir_entry_len);
+  return (len);
 }
 
 
@@ -841,18 +782,11 @@ ssh_exec (gftp_request * request, const char *command, size_t len)
       switch (ret)
         {
           case -1: 
+            g_free (message.data);
             ssh_disconnect (request);
             return (message.data);
           case DATA:
           case ERROR:
-            for (pos = (char *) message.data; 
-                 pos < (char *) message.data+message.len-1; 
-                 pos++)
-              {
-                if (*pos == '\0')
-                  *pos = '\0';
-              }
-
             pos = (char *) message.data+message.len-1;
             if (*pos == '\n')
               *pos = '\0';
@@ -1056,9 +990,9 @@ static int
 ssh_send_command (gftp_request * request, int cmdnum, const char *command, 
                   size_t len)
 {
+  ssize_t wrote, numleft;
   ssh_parms * params;
-  ssize_t wrote;
-  char *buf;
+  char *buf, *pos;
   int clen;
 
   params = request->protocol_data;
@@ -1071,17 +1005,36 @@ ssh_send_command (gftp_request * request, int cmdnum, const char *command,
     memcpy (&buf[6], command, len);
   ssh_log_command (request, params->channel, cmdnum, command, len, 1);
 
-  wrote = fwrite (buf, 1, len + 6, request->sockfd_write);
+  numleft = len + 6;
+  pos = buf;
+
+  while (numleft > 0)
+    {
+      wrote = fwrite (pos, 1, numleft, request->sockfd_write);
+      if (ferror (request->sockfd_write))
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                                   _("Error: Could not write to socket: %s\n"),
+                                   g_strerror (errno));
+          ssh_disconnect (request);
+          g_free (buf);
+          return (-1);
+        }
+      numleft -= wrote;
+      pos += wrote;
+    }
+
+  g_free (buf);
+
+  fflush (request->sockfd_write);
   if (ferror (request->sockfd_write))
     {
       request->logging_function (gftp_logging_error, request->user_data,
-				 _("Error: Could not write to socket: %s\n"),
+                                 _("Error: Could not write to socket: %s\n"),
                                  g_strerror (errno));
       ssh_disconnect (request);
       return (-1);
     }
-  fflush (request->sockfd_write);
-  g_free (buf);
   return 0;
 
 }
@@ -1284,148 +1237,6 @@ ssh_log_command (gftp_request * request, int channel, int cmdnum,
 }
 
 
-static int
-ptym_open (char *pts_name)
-{
-
-#if defined(SYSV)
-
-  char *tempstr;
-  int fd;
-
-  strcpy (pts_name, "/dev/ptmx");
-  if ((fd = open (pts_name, O_RDWR)) < 0)
-    return (-1);
-
-  if (grantpt (fd) < 0)
-    {
-      close (fd);
-      return (-1);
-    }
-
-  if (unlockpt (fd) < 0)
-    {
-      close (fd);
-      return (-1);
-    }
-
-  if ((tempstr = ptsname (fd)) == NULL)
-    {
-      close (fd);
-      return (-1);
-    }
-
-  strcpy (pts_name, tempstr);
-  return (fd);
-
-#else
-
-  char *pos1, *pos2;
-  int fd;
-
-  strcpy (pts_name, "/dev/ptyXY");
-  for (pos1 = "pqrstuvwxyzPQRST"; *pos1 != '\0'; pos1++) 
-    {
-      pts_name[8] = *pos1;
-      for (pos2 = "0123456789abcdef"; *pos2 != '\0'; pos2++)
-        {
-          pts_name[9] = *pos2;
-          if ((fd = open (pts_name, O_RDWR)) < 0)
-            {
-              if (errno == ENOENT)
-                return (-1);
-              else
-                continue;
-            }
-          pts_name[5] = 't';
-          return (fd);
-        }
-    }
-  return (-1);
-
-#endif
-
-}
-
-
-static int
-ptys_open (int fdm, char *pts_name)
-{
-  int fds;
-
-#if !defined(SYSV)
-
-  chmod (pts_name, S_IRUSR | S_IWUSR);
-  chown (pts_name, getuid (), -1);
-
-#endif
-
-  if ((fds = open (pts_name, O_RDWR)) < 0)
-    {
-      close (fdm);
-      return (-1);
-    }
-
-#if defined(SYSV)
-
-  if (ioctl (fds, I_PUSH, "ptem") < 0)
-    {
-      close (fdm);
-      close (fds);
-      return (-1);
-    }
-
-  if (ioctl (fds, I_PUSH, "ldterm") < 0)
-    {
-      close (fdm);
-      close (fds);
-      return (-1);
-    }
-
-  if (ioctl (fds, I_PUSH, "ttcompat") < 0)
-    {
-      close (fdm);
-      close (fds);
-      return (-1);
-    }
-
-#endif
-
-#if !defined(SYSV) && defined(TIOCSCTTY) && !defined(CIBAUD)
-
-  if (ioctl (fds, TIOCSCTTY, (char *) 0) < 0)
-    {
-      close (fdm);
-      return (-1);
-    }
-
-#endif
-
-  return (fds);
-}
-
-
-static int
-tty_raw (int fd)
-{
-  struct termios buf;
-
-  if (tcgetattr (fd, &buf) < 0)
-    return (-1);
-
-  buf.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
-  buf.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
-  buf.c_cflag &= ~(CSIZE | PARENB);
-  buf.c_cflag |= CS8;
-  buf.c_oflag &= ~(OPOST);
-  buf.c_cc[VMIN] = 1;
-  buf.c_cc[VTIME] = 0;
-
-  if (tcsetattr (fd, TCSANOW, &buf) < 0)
-    return (-1);
-  return (0);
-}
-
 static size_t 
 ssh_remove_spaces ( char *string )
 {
