@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  bookmarks.c - routines for the bookmarks                                 */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -54,7 +54,7 @@ static void clear_bookmarks_tree 		( void );
 
 static GtkWidget * bm_hostedit, * bm_portedit, * bm_localdiredit,
   * bm_remotediredit, * bm_useredit, * bm_passedit, * bm_acctedit, * anon_chk,
-  * bm_pathedit, * bm_protocol, * tree;
+  * bm_pathedit, * bm_protocol, * tree, *bm_sftppath;
 static GHashTable * new_bookmarks_htable;
 static gftp_bookmarks * new_bookmarks;
 static GtkItemFactory * edit_factory;
@@ -109,6 +109,7 @@ run_bookmark (gpointer data)
   gftp_set_hostname (current_wdata->request, tempentry->hostname);
   gftp_set_directory (current_wdata->request, tempentry->remote_dir);
   gftp_set_port (current_wdata->request, tempentry->port);
+  gftp_set_sftpserv_path (current_wdata->request, tempentry->sftpserv_path);
 
   for (i = 0; gftp_protocols[i].name; i++)
     {
@@ -128,9 +129,9 @@ run_bookmark (gpointer data)
 void
 add_bookmark (gpointer data)
 {
-  char *edttxt;
+  const char *edttxt;
 
-  if (!check_status (_("Add Bookmark"), current_wdata, 0, 0, 1))
+  if (!check_status (_("Add Bookmark"), current_wdata, 0, 0, 0, 1))
     return;
 
   edttxt = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry));
@@ -149,8 +150,9 @@ static void
 doadd_bookmark (GtkWidget * widget, gftp_dialog_data * data)
 {
   GtkItemFactoryEntry test = { NULL, NULL, run_bookmark, 0 };
+  const char *edttxt, *spos;
   gftp_bookmarks * tempentry;
-  char *edttxt;
+  char *dpos;
 
   edttxt = gtk_entry_get_text (GTK_ENTRY (data->edit));
   if (*edttxt == '\0')
@@ -168,8 +170,18 @@ doadd_bookmark (GtkWidget * widget, gftp_dialog_data * data)
     }
 
   tempentry = g_malloc0 (sizeof (*tempentry));
-  tempentry->path = g_malloc (strlen (edttxt) + 1);
-  strcpy (tempentry->path, edttxt);
+
+  dpos = tempentry->path = g_malloc (strlen (edttxt) + 1);
+  for (spos = edttxt; *spos != '\0';)
+    {
+      *dpos++ = *spos++;
+      if (*spos == '/')
+        {
+          *dpos++ = '/';
+          for (; *spos == '/'; spos++);
+        }
+    }
+  *dpos = '\0';
 
   edttxt = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry));
   tempentry->hostname = g_malloc (strlen (edttxt) + 1);
@@ -277,6 +289,7 @@ edit_bookmarks (gpointer data)
 
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Bookmarks"));
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "Edit Bookmarks", "gFTP");
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 15);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 
@@ -290,7 +303,12 @@ edit_bookmarks (gpointer data)
   scroll = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (scroll, 150, 200);
+#else
+  gtk_widget_set_size_request (scroll, 150, 200);
+#endif
+
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), scroll, TRUE, TRUE,
 		      0);
   gtk_container_border_width (GTK_CONTAINER (scroll), 3);
@@ -496,6 +514,11 @@ copy_bookmarks (gftp_bookmarks * bookmarks)
 	  newentry->acct = g_malloc (strlen (tempentry->acct) + 1);
 	  strcpy (newentry->acct, tempentry->acct);
 	}
+      if (tempentry->sftpserv_path)
+        {
+          newentry->sftpserv_path = g_malloc (strlen (tempentry->sftpserv_path) + 1);
+          strcpy (newentry->sftpserv_path, tempentry->sftpserv_path);
+        }
       newentry->port = tempentry->port;
 
       if (sibling == NULL)
@@ -713,19 +736,21 @@ do_make_new (GtkWidget * widget, gftp_dialog_data * data)
   GdkPixmap * closedir_pixmap, * opendir_pixmap;
   GdkBitmap * closedir_bitmap, * opendir_bitmap;  
   gftp_bookmarks * tempentry, * newentry;
-  char *str, *pos, *text[2];
   GtkCTreeNode * sibling;
+  char *pos, *text[2];
+  const char *str;
 
   gftp_get_pixmap (tree, "open_dir.xpm", &opendir_pixmap, &opendir_bitmap);
   gftp_get_pixmap (tree, "dir.xpm", &closedir_pixmap, &closedir_bitmap);
 
-  pos = str = gtk_entry_get_text (GTK_ENTRY (data->edit));
-  while ((pos = strchr (pos, '/')) != NULL)
-    *pos++ = ' ';
+  str = gtk_entry_get_text (GTK_ENTRY (data->edit));
 
   newentry = g_malloc0 (sizeof (*newentry));
   newentry->path = g_malloc (strlen (str) + 1);
   strcpy (newentry->path, str);
+  while ((pos = strchr (str, '/')) != NULL)
+    *pos++ = ' ';
+
   newentry->prev = new_bookmarks;
   if (data->data)
     newentry->isfolder = 1;
@@ -782,6 +807,7 @@ edit_entry (gpointer data)
 
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Entry"));
+  gtk_window_set_wmclass (GTK_WINDOW (dialog), "Edit Bookmark Entry", "gFTP");
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 15);
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
@@ -893,17 +919,30 @@ edit_entry (gpointer data)
     gtk_entry_set_text (GTK_ENTRY (bm_localdiredit), entry->local_dir);
   gtk_widget_show (bm_localdiredit);
 
+  tempwid = gtk_label_new (_("Remote SSH sftp path:"));
+  gtk_misc_set_alignment (GTK_MISC (tempwid), 1, 0.5);
+  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 6, 7);
+  gtk_widget_show (tempwid);
+
+  bm_sftppath = gtk_entry_new ();
+  gtk_table_attach_defaults (GTK_TABLE (table), bm_sftppath, 1, 2, 6, 7);
+  if (entry->isfolder)
+    gtk_widget_set_sensitive (bm_sftppath, 0);
+  else if (entry->sftpserv_path)
+    gtk_entry_set_text (GTK_ENTRY (bm_sftppath), entry->sftpserv_path);
+  gtk_widget_show (bm_sftppath);
+
   tempwid = gtk_hseparator_new ();
-  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 2, 6, 7);
+  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 2, 7, 8);
   gtk_widget_show (tempwid);
 
   tempwid = gtk_label_new (_("Username:"));
   gtk_misc_set_alignment (GTK_MISC (tempwid), 1, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 7, 8);
+  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 8, 9);
   gtk_widget_show (tempwid);
 
   bm_useredit = gtk_entry_new ();
-  gtk_table_attach_defaults (GTK_TABLE (table), bm_useredit, 1, 2, 7, 8);
+  gtk_table_attach_defaults (GTK_TABLE (table), bm_useredit, 1, 2, 8, 9);
   if (entry->isfolder)
     gtk_widget_set_sensitive (bm_useredit, 0);
   else if (entry->user)
@@ -912,11 +951,11 @@ edit_entry (gpointer data)
 
   tempwid = gtk_label_new (_("Password:"));
   gtk_misc_set_alignment (GTK_MISC (tempwid), 1, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 8, 9);
+  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 9, 10);
   gtk_widget_show (tempwid);
 
   bm_passedit = gtk_entry_new ();
-  gtk_table_attach_defaults (GTK_TABLE (table), bm_passedit, 1, 2, 8, 9);
+  gtk_table_attach_defaults (GTK_TABLE (table), bm_passedit, 1, 2, 9, 10);
   gtk_entry_set_visibility (GTK_ENTRY (bm_passedit), FALSE);
   if (entry->isfolder)
     gtk_widget_set_sensitive (bm_passedit, 0);
@@ -926,11 +965,11 @@ edit_entry (gpointer data)
 
   tempwid = gtk_label_new (_("Account:"));
   gtk_misc_set_alignment (GTK_MISC (tempwid), 1, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 9, 10);
+  gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 0, 1, 10, 11);
   gtk_widget_show (tempwid);
 
   bm_acctedit = gtk_entry_new ();
-  gtk_table_attach_defaults (GTK_TABLE (table), bm_acctedit, 1, 2, 9, 10);
+  gtk_table_attach_defaults (GTK_TABLE (table), bm_acctedit, 1, 2, 10, 11);
   gtk_entry_set_visibility (GTK_ENTRY (bm_acctedit), FALSE);
   if (entry->isfolder)
     gtk_widget_set_sensitive (bm_acctedit, 0);
@@ -939,7 +978,7 @@ edit_entry (gpointer data)
   gtk_widget_show (bm_acctedit);
 
   anon_chk = gtk_check_button_new_with_label (_("Log in as ANONYMOUS"));
-  gtk_table_attach_defaults (GTK_TABLE (table), anon_chk, 0, 2, 10, 11);
+  gtk_table_attach_defaults (GTK_TABLE (table), anon_chk, 0, 2, 11, 12);
   if (entry->isfolder)
     gtk_widget_set_sensitive (anon_chk, 0);
   else
@@ -1052,6 +1091,8 @@ do_delete_entry (GtkWidget * widget, gftp_dialog_data * data)
 	g_free (tempentry->user);
       if (tempentry->pass)
 	g_free (tempentry->pass);
+      if (tempentry->sftpserv_path)
+        g_free (tempentry->sftpserv_path);
 
       if (tempentry->children != NULL)
 	{
@@ -1077,8 +1118,9 @@ static void
 entry_apply_changes (GtkWidget * widget, gftp_bookmarks * entry)
 {
   gftp_bookmarks * tempentry, * nextentry;
-  char *str, *pos, *newpath, tempchar;
+  char *pos, *newpath, tempchar, *tempstr;
   GtkWidget * tempwid;
+  const char *str;
   int oldpathlen;
 
   oldpathlen = strlen (entry->path);
@@ -1127,6 +1169,17 @@ entry_apply_changes (GtkWidget * widget, gftp_bookmarks * entry)
   entry->local_dir = g_malloc (strlen (str) + 1);
   strcpy (entry->local_dir, str);
 
+  str = gtk_entry_get_text (GTK_ENTRY (bm_sftppath));
+  if (entry->sftpserv_path)
+    g_free (entry->sftpserv_path);
+  if (strlen (str) > 0)
+    {
+      entry->sftpserv_path = g_malloc (strlen (str) + 1);
+      strcpy (entry->sftpserv_path, str);
+    }
+  else
+    entry->sftpserv_path = NULL;
+
   if (GTK_TOGGLE_BUTTON (anon_chk)->active)
     str = "anonymous";
   else
@@ -1163,10 +1216,11 @@ entry_apply_changes (GtkWidget * widget, gftp_bookmarks * entry)
       while (tempentry != NULL)
 	{
 	  g_hash_table_remove (new_bookmarks_htable, tempentry->path);
-	  str = g_strconcat (newpath, "/", tempentry->path + oldpathlen, NULL);
-	  remove_double_slashes (str);
+	  tempstr = g_strconcat (newpath, "/", tempentry->path + oldpathlen, 
+                                 NULL);
+	  remove_double_slashes (tempstr);
 	  g_free (tempentry->path);
-	  tempentry->path = str;
+	  tempentry->path = tempstr;
 	  g_hash_table_insert (new_bookmarks_htable, tempentry->path,
 			       tempentry);
 	  if (tempentry->children != NULL)
