@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-gtk.c - GTK+ 1.2 port of gftp                                       */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -24,7 +24,7 @@ static gint delete_event 			( GtkWidget * widget,
 						  gpointer data );
 static void destroy 				( GtkWidget * widget, 
 						  gpointer data );
-static void sig_child 				( int signo );
+static RETSIGTYPE sig_child 			( int signo );
 static GtkWidget * CreateFTPWindows 		( GtkWidget * ui );
 static GtkWidget * CreateMenus			( GtkWidget * parent );
 static GtkWidget * CreateToolbar 		( GtkWidget * parent );
@@ -61,6 +61,7 @@ static GtkWidget * local_frame, * remote_frame, * log_table, * transfer_scroll,
 gftp_window_data window1, window2, *other_wdata, *current_wdata;
 GtkWidget * stop_btn, * hostedit, * useredit, * passedit, * portedit, * logwdw,
           * dlwdw, * protocol_menu, * optionmenu;
+GtkAdjustment * logwdw_vadj;
 int local_start, remote_start, trans_start, log_start, tools_start;
 GHashTable * graphic_hash_table = NULL;
 GtkItemFactoryEntry * menus = NULL;
@@ -72,6 +73,7 @@ int
 main (int argc, char *argv[])
 {
   GtkWidget *window, *ui;
+  gftp_graphic * graphic;
 
 #ifdef HAVE_GETTEXT
   setlocale (LC_ALL, "");
@@ -95,9 +97,19 @@ main (int argc, char *argv[])
   gtk_signal_connect (GTK_OBJECT (window), "destroy",
 		      GTK_SIGNAL_FUNC (destroy), NULL);
   gtk_window_set_title (GTK_WINDOW (window), version);
+  gtk_window_set_wmclass (GTK_WINDOW(window), "main", "gFTP");
+  gtk_widget_set_name (window, version);
   gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
   gtk_widget_realize (window);
 
+  graphic = open_xpm (window, "gftp.xpm");
+  if (graphic != NULL)
+    {
+      gdk_window_set_icon (window->window, NULL, graphic->pixmap,
+                           graphic->bitmap);
+      gdk_window_set_icon_name (window->window, _("gFTP Icon"));
+    }
+
   other_wdata = &window1;
   current_wdata = &window2;
   ui = CreateFTPWindows (window);
@@ -105,7 +117,7 @@ main (int argc, char *argv[])
   gtk_widget_show (window);
 
   ftp_log (gftp_logging_misc, NULL,
-	   "%s, Copyright (C) 1998-2001 Brian Masney <", version);
+	   "%s, Copyright (C) 1998-2002 Brian Masney <", version);
   ftp_log (gftp_logging_recv, NULL, "masneyb@seul.org");
   ftp_log (gftp_logging_misc, NULL,
 	   _(">. If you have any questions, comments, or suggestions about this program, please feel free to email them to me. You can always find out the latest news about gFTP from my website at http://gftp.seul.org/\n"));
@@ -149,7 +161,7 @@ destroy (GtkWidget * widget, gpointer data)
 }
 
 
-static void
+static RETSIGTYPE
 sig_child (int signo)
 {
   viewedit_process_done = 1;
@@ -162,6 +174,11 @@ CreateFTPWindows (GtkWidget * ui)
   GtkWidget *box, *dlbox, *winpane, *dlpane, *logpane, *mainvbox, *tempwid,
             *button;
   char *dltitles[2];
+#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+  GtkTextBuffer * textbuf;
+  GtkTextTag *tag;
+  GdkColor fore;
+#endif
 
   memset (&window1, 0, sizeof (window1));
   memset (&window2, 0, sizeof (window2));
@@ -174,26 +191,20 @@ CreateFTPWindows (GtkWidget * ui)
 
   tempwid = CreateMenus (ui);
   gtk_box_pack_start (GTK_BOX (mainvbox), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   tempwid = CreateToolbar (ui);
   gtk_box_pack_start (GTK_BOX (mainvbox), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   winpane = gtk_hpaned_new ();
-  gtk_widget_show (winpane);
 
   box = gtk_hbox_new (FALSE, 0);
-  gtk_widget_show (box);
 
   local_frame = CreateFTPWindow (&window1, listbox_local_width, local_columns);
   gtk_box_pack_start (GTK_BOX (box), local_frame, TRUE, TRUE, 0);
-  gtk_widget_show (local_frame);
 
   dlbox = gtk_vbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (dlbox), 5);
   gtk_box_pack_start (GTK_BOX (box), dlbox, FALSE, FALSE, 0);
-  gtk_widget_show (dlbox);
 
   tempwid = toolbar_pixmap (ui, "right.xpm");
   button = gtk_button_new ();
@@ -201,7 +212,6 @@ CreateFTPWindows (GtkWidget * ui)
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (put_files), NULL);
   gtk_container_add (GTK_CONTAINER (button), tempwid);
-  gtk_widget_show (button);
 
   tempwid = toolbar_pixmap (ui, "left.xpm");
   button = gtk_button_new ();
@@ -209,21 +219,22 @@ CreateFTPWindows (GtkWidget * ui)
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
 			     GTK_SIGNAL_FUNC (get_files), NULL);
   gtk_container_add (GTK_CONTAINER (button), tempwid);
-  gtk_widget_show (button);
 
   gtk_paned_pack1 (GTK_PANED (winpane), box, 1, 1);
 
   remote_frame = CreateFTPWindow (&window2, listbox_remote_width, 
                                   remote_columns);
   gtk_paned_pack2 (GTK_PANED (winpane), remote_frame, 1, 1);
-  gtk_widget_show (remote_frame);
 
   dlpane = gtk_vpaned_new ();
   gtk_paned_pack1 (GTK_PANED (dlpane), winpane, 1, 1);
-  gtk_widget_show (dlpane);
 
   transfer_scroll = gtk_scrolled_window_new (NULL, NULL);
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (transfer_scroll, 1, transfer_height);
+#else
+  gtk_widget_set_size_request (transfer_scroll, -1, transfer_height);
+#endif
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (transfer_scroll),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
@@ -238,40 +249,84 @@ CreateFTPWindows (GtkWidget * ui)
   else
     gtk_clist_set_column_width (GTK_CLIST (dlwdw), 0, file_trans_column);
 
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
-					 (transfer_scroll), dlwdw);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (transfer_scroll),                                          dlwdw);
   gtk_signal_connect (GTK_OBJECT (dlwdw), "button_press_event",
 		      GTK_SIGNAL_FUNC (menu_mouse_click), (gpointer) dl_factory);
   gtk_paned_pack2 (GTK_PANED (dlpane), transfer_scroll, 1, 1);
-  gtk_widget_show (dlwdw);
-  gtk_widget_show (transfer_scroll);
 
   logpane = gtk_vpaned_new ();
   gtk_paned_pack1 (GTK_PANED (logpane), dlpane, 1, 1);
-  gtk_widget_show (logpane);
 
   log_table = gtk_table_new (1, 2, FALSE);
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (log_table, 1, log_height);
+
   logwdw = gtk_text_new (NULL, NULL);
+
   gtk_text_set_editable (GTK_TEXT (logwdw), FALSE);
   gtk_text_set_word_wrap (GTK_TEXT (logwdw), TRUE);
+
   gtk_table_attach (GTK_TABLE (log_table), logwdw, 0, 1, 0, 1,
 		    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		    0, 0);
   gtk_signal_connect (GTK_OBJECT (logwdw), "button_press_event",
-		      GTK_SIGNAL_FUNC (menu_mouse_click), (gpointer) log_factory);
-  gtk_widget_show (logwdw);
+		      GTK_SIGNAL_FUNC (menu_mouse_click), 
+                      (gpointer) log_factory);
 
   tempwid = gtk_vscrollbar_new (GTK_TEXT (logwdw)->vadj);
   gtk_table_attach (GTK_TABLE (log_table), tempwid, 1, 2, 0, 1,
 		    GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
-  gtk_widget_show (tempwid);
-  gtk_widget_show (log_table);
+  logwdw_vadj = GTK_TEXT (logwdw)->vadj;
+#else
+  gtk_widget_set_size_request (log_table, -1, log_height);
+
+  logwdw = gtk_text_view_new ();
+  gtk_text_view_set_editable (GTK_TEXT_VIEW (logwdw), FALSE);
+  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (logwdw), GTK_WRAP_WORD);
+
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
+
+  tag = gtk_text_buffer_create_tag (textbuf, "send", NULL);
+  fore.red = send_color.red;
+  fore.green = send_color.green;
+  fore.blue = send_color.blue;
+  g_object_set (G_OBJECT (tag), "foreground_gdk", &fore, NULL);
+
+  tag = gtk_text_buffer_create_tag (textbuf, "recv", NULL);
+  fore.red = recv_color.red;
+  fore.green = recv_color.green;
+  fore.blue = recv_color.blue;
+  g_object_set (G_OBJECT (tag), "foreground_gdk", &fore, NULL);
+
+  tag = gtk_text_buffer_create_tag (textbuf, "error", NULL);
+  fore.red = error_color.red;
+  fore.green = error_color.green;
+  fore.blue = error_color.blue;
+  g_object_set (G_OBJECT (tag), "foreground_gdk", &fore, NULL);
+
+  tag = gtk_text_buffer_create_tag (textbuf, "misc", NULL);
+  fore.red = misc_color.red;
+  fore.green = misc_color.green;
+  fore.blue = misc_color.blue;
+  g_object_set (G_OBJECT (tag), "foreground_gdk", &fore, NULL);
+
+  tempwid = gtk_scrolled_window_new (NULL, NULL);
+  logwdw_vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (tempwid));
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (tempwid),
+                                 GTK_POLICY_NEVER,
+                                 GTK_POLICY_AUTOMATIC);
+
+  gtk_container_add (GTK_CONTAINER (tempwid), logwdw);
+  gtk_table_attach (GTK_TABLE (log_table), tempwid, 0, 1, 0, 1,
+		    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
+		    0, 0);
+#endif
 
   gtk_paned_pack2 (GTK_PANED (logpane), log_table, 1, 1);
   gtk_box_pack_start (GTK_BOX (mainvbox), logpane, TRUE, TRUE, 0);
 
-  gtk_widget_show (mainvbox);
+  gtk_widget_show_all (mainvbox);
   return (mainvbox);
 }
 
@@ -414,7 +469,13 @@ CreateMenus (GtkWidget * parent)
 
   build_bookmarks_menu ();
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_accel_group_attach (accel_group, GTK_OBJECT (parent));
+#else
+  /* FIXME 
+  gtk_accel_group_attach (accel_group, G_OBJECT (parent));
+*/
+#endif
 
   tempwid = gtk_item_factory_get_widget (factory, _(menu_items[6].path));
   gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (tempwid), TRUE);
@@ -471,14 +532,13 @@ CreateToolbar (GtkWidget * parent)
     {"application/x-rootwin-drop", 0, 1}
   };
   GtkWidget *toolbar, *box, *tempwid;
-  int i;
+  int i, num;
 
   toolbar = gtk_handle_box_new ();
 
   box = gtk_hbox_new (FALSE, 4);
   gtk_container_add (GTK_CONTAINER (toolbar), box);
   gtk_container_border_width (GTK_CONTAINER (box), 5);
-  gtk_widget_show (box);
 
   tempwid = toolbar_pixmap (parent, "connect.xpm");
   openurl_btn = gtk_button_new ();
@@ -491,15 +551,19 @@ CreateToolbar (GtkWidget * parent)
 		     GDK_ACTION_COPY | GDK_ACTION_MOVE);
   gtk_container_border_width (GTK_CONTAINER (openurl_btn), 1);
   gtk_box_pack_start (GTK_BOX (box), openurl_btn, FALSE, FALSE, 0);
-  gtk_widget_show (openurl_btn);
 
   tempwid = gtk_label_new (_("Host: "));
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   hostedit = gtk_combo_new ();
   gtk_combo_set_case_sensitive (GTK_COMBO (hostedit), 1);
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (hostedit, 130, -1);
+#else
+  gtk_widget_set_size_request (hostedit, 130, -1);
+#endif
+
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (hostedit)->entry), "activate",
 		      GTK_SIGNAL_FUNC (toolbar_hostedit), NULL);
   if (host_history)
@@ -507,15 +571,19 @@ CreateToolbar (GtkWidget * parent)
   gtk_combo_disable_activate (GTK_COMBO (hostedit));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry), "");
   gtk_box_pack_start (GTK_BOX (box), hostedit, TRUE, TRUE, 0);
-  gtk_widget_show (hostedit);
 
   tempwid = gtk_label_new (_("Port: "));
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   portedit = gtk_combo_new ();
   gtk_combo_set_case_sensitive (GTK_COMBO (portedit), 1);
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (portedit, 40, -1);
+#else
+  gtk_widget_set_size_request (portedit, 40, -1);
+#endif
+
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (portedit)->entry), "activate",
 		      GTK_SIGNAL_FUNC (toolbar_hostedit), NULL);
   if (port_history)
@@ -523,15 +591,19 @@ CreateToolbar (GtkWidget * parent)
   gtk_combo_disable_activate (GTK_COMBO (portedit));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), "");
   gtk_box_pack_start (GTK_BOX (box), portedit, FALSE, FALSE, 0);
-  gtk_widget_show (portedit);
 
   tempwid = gtk_label_new (_("User: "));
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   useredit = gtk_combo_new ();
   gtk_combo_set_case_sensitive (GTK_COMBO (useredit), 1);
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (useredit, 75, -1);
+#else
+  gtk_widget_set_size_request (useredit, 75, -1);
+#endif
+
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (useredit)->entry), "activate",
 		      GTK_SIGNAL_FUNC (toolbar_hostedit), NULL);
   if (user_history)
@@ -539,38 +611,43 @@ CreateToolbar (GtkWidget * parent)
   gtk_combo_disable_activate (GTK_COMBO (useredit));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (useredit)->entry), "");
   gtk_box_pack_start (GTK_BOX (box), useredit, TRUE, TRUE, 0);
-  gtk_widget_show (useredit);
 
   tempwid = gtk_label_new (_("Pass: "));
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   passedit = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (passedit, 55, -1);
+#else
+  gtk_widget_set_size_request (passedit, 55, -1);
+#endif
+
   gtk_entry_set_visibility (GTK_ENTRY (passedit), FALSE);
   gtk_signal_connect (GTK_OBJECT (passedit), "activate",
 		      GTK_SIGNAL_FUNC (toolbar_hostedit), NULL);
   gtk_box_pack_start (GTK_BOX (box), passedit, FALSE, FALSE, 0);
-  gtk_widget_show (passedit);
 
   tempwid = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
-  gtk_widget_show (tempwid);
 
   optionmenu = gtk_option_menu_new ();
   gtk_box_pack_start (GTK_BOX (tempwid), optionmenu, TRUE, FALSE, 0);
-  gtk_widget_show (optionmenu);
 
+  num = 0;
   protocol_menu = gtk_menu_new ();
   for (i = 0; gftp_protocols[i].name; i++)
     {
+      if (strcmp (gftp_protocols[i].name, default_protocol) == 0)
+        num = i;
+
       tempwid = gtk_menu_item_new_with_label (gftp_protocols[i].name);
       gtk_object_set_user_data (GTK_OBJECT (tempwid), (gpointer) i);
       gtk_menu_append (GTK_MENU (protocol_menu), tempwid);
       gtk_widget_show (tempwid);
     }
   gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), protocol_menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), 0);
+  gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), num);
 
   tempwid = toolbar_pixmap (parent, "stop.xpm");
   stop_btn = gtk_button_new ();
@@ -580,7 +657,6 @@ CreateToolbar (GtkWidget * parent)
 			     GTK_SIGNAL_FUNC (stop_button), NULL);
   gtk_container_border_width (GTK_CONTAINER (stop_btn), 1);
   gtk_box_pack_start (GTK_BOX (box), stop_btn, FALSE, FALSE, 0);
-  gtk_widget_show (stop_btn);
 
   return (toolbar);
 }
@@ -693,9 +769,14 @@ CreateFTPWindow (gftp_window_data * wdata, int width, int columns[6])
   wdata->sortasds = 1;
 
   parent = gtk_frame_new (NULL);
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (parent, width, listbox_file_height);
+#else
+  gtk_widget_set_size_request (parent, width, listbox_file_height);
+#endif
+
   gtk_container_border_width (GTK_CONTAINER (parent), 5);
-  gtk_widget_show (parent);
 
   box = gtk_vbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box), 5);
@@ -711,12 +792,10 @@ CreateFTPWindow (gftp_window_data * wdata, int width, int columns[6])
     gtk_combo_set_popdown_strings (GTK_COMBO (wdata->combo), *wdata->history);
   gtk_combo_disable_activate (GTK_COMBO (wdata->combo));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry), "");
-  gtk_widget_show (wdata->combo);
 
   wdata->hoststxt = gtk_label_new (NULL);
   gtk_misc_set_alignment (GTK_MISC (wdata->hoststxt), 0, 0);
   gtk_box_pack_start (GTK_BOX (box), wdata->hoststxt, FALSE, FALSE, 0);
-  gtk_widget_show (wdata->hoststxt);
 
   scroll_list = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_list),
@@ -727,8 +806,15 @@ CreateFTPWindow (gftp_window_data * wdata, int width, int columns[6])
 		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
   gtk_drag_dest_set (wdata->listbox, GTK_DEST_DEFAULT_ALL, possible_types, 2,
 		     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_clist_set_selection_mode (GTK_CLIST (wdata->listbox),
 				GTK_SELECTION_EXTENDED);
+#else
+  gtk_clist_set_selection_mode (GTK_CLIST (wdata->listbox),
+				GTK_SELECTION_MULTIPLE);
+#endif
+
   gtk_clist_set_column_width (GTK_CLIST (wdata->listbox), 0, 16);
   gtk_clist_set_column_justification (GTK_CLIST (wdata->listbox), 0,
 				      GTK_JUSTIFY_CENTER);
@@ -743,9 +829,15 @@ CreateFTPWindow (gftp_window_data * wdata, int width, int columns[6])
   gtk_box_pack_start (GTK_BOX (box), scroll_list, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (wdata->listbox), "click_column",
 		      GTK_SIGNAL_FUNC (sortrows), (gpointer) wdata);
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_signal_connect_after (GTK_OBJECT (wdata->listbox), "button_press_event",
 			    GTK_SIGNAL_FUNC (list_dblclick),
 			    (gpointer) wdata);
+#else
+  gtk_signal_connect (GTK_OBJECT (wdata->listbox), "button_press_event",
+			    GTK_SIGNAL_FUNC (list_dblclick),
+			    (gpointer) wdata);
+#endif
   gtk_signal_connect (GTK_OBJECT (wdata->listbox), "key_press_event",
 		      GTK_SIGNAL_FUNC (list_enter), (gpointer) wdata);
   gtk_signal_connect (GTK_OBJECT (wdata->listbox), "drag_data_get",
@@ -753,9 +845,6 @@ CreateFTPWindow (gftp_window_data * wdata, int width, int columns[6])
   gtk_signal_connect (GTK_OBJECT (wdata->listbox), "drag_data_received",
 		      GTK_SIGNAL_FUNC (listbox_get_drag_data),
 		      (gpointer) wdata);
-  gtk_widget_show (wdata->listbox);
-  gtk_widget_show (scroll_list);
-  gtk_widget_show (box);
   return (parent);
 }
 
@@ -883,7 +972,7 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
 {
   void (*init) (gftp_request * request);
   GtkWidget *tempwid;
-  char *txt;
+  const char *txt;
   int num;
 
   if (current_wdata->request->stopable)
@@ -899,16 +988,23 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
   init = gftp_protocols[num].init;
   init (current_wdata->request);
 
-  txt = alltrim (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry)));
-  if (current_wdata->request->need_hostport && *txt == '\0')
+  if (GFTP_IS_CONNECTED (current_wdata->request))
+    disconnect (current_wdata);
+
+  gftp_set_hostname (current_wdata->request, 
+                gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry)));
+  alltrim (current_wdata->request->hostname);
+
+  if (current_wdata->request->need_hostport && 
+      *current_wdata->request->hostname == '\0')
     {
       ftp_log (gftp_logging_error, NULL,
 	       _("Error: You must type in a host to connect to\n"));
       return;
     }
 
-  if (GFTP_IS_CONNECTED (current_wdata->request))
-    disconnect (current_wdata);
+  add_history (hostedit, &host_history, &host_len, 
+               current_wdata->request->hostname);
 
   if (current_wdata->request->need_userpass &&
       *gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (useredit)->entry)) == '\0')
@@ -917,24 +1013,24 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
        gftp_set_password (current_wdata->request, emailaddr);
     }
 
-  gftp_set_hostname (current_wdata->request, txt);
-  add_history (hostedit, &host_history, &host_len, txt);
 
-  txt = alltrim (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (portedit)->entry)));
+  txt = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (portedit)->entry));
   gftp_set_port (current_wdata->request, strtol (txt, NULL, 10));
   add_history (portedit, &port_history, &port_len, txt);
 
-  txt = alltrim (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (useredit)->entry)));
-  gftp_set_username (current_wdata->request, txt);
-  add_history (useredit, &user_history, &user_len, txt);
+  gftp_set_username (current_wdata->request, gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (useredit)->entry)));
+  alltrim (current_wdata->request->username);
+  add_history (useredit, &user_history, &user_len, 
+               current_wdata->request->username);
 
   gftp_set_password (current_wdata->request,
 		     gtk_entry_get_text (GTK_ENTRY (passedit)));
 
-  txt = alltrim (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (current_wdata->combo)->entry)));
-  gftp_set_directory (current_wdata->request, txt);
+  gftp_set_directory (current_wdata->request, gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (current_wdata->combo)->entry)));
+  alltrim (current_wdata->request->directory);
+
   add_history (current_wdata->combo, current_wdata->history, 
-               current_wdata->histlen, txt);
+               current_wdata->histlen, current_wdata->request->directory);
 
   ftp_connect (current_wdata, current_wdata->request, 1);
 }
@@ -953,7 +1049,7 @@ sortrows (GtkCList * clist, gint column, gpointer data)
   if (wdata->files == NULL)
     return;
 
-  if (!check_status (_("Sort"), wdata, 0, 0, 1))
+  if (!check_status (_("Sort"), wdata, 0, 0, 0, 1))
     return;
 
   gtk_label_set (GTK_LABEL (wdata->hoststxt), _("Sorting..."));
