@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  menu-items.c - menu callbacks                                            */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -31,6 +31,9 @@ static void *do_change_dir_thread 		( void * data );
 static void dosavelog 				( GtkWidget * widget, 
 						  GtkFileSelection * fs );
 
+static RETSIGTYPE sig_chdirquit                 ( int signo );
+static sigjmp_buf chdirenvir;
+
 void
 change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
 {
@@ -63,7 +66,7 @@ change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
 void
 tb_openurl_dialog (gpointer data)
 {
-  char *edttxt;
+  const char *edttxt;
 
   if (current_wdata->request->stopable)
     {
@@ -103,7 +106,7 @@ static void
 do_openurl (GtkWidget * widget, gftp_dialog_data * data)
 {
   gftp_window_data * wdata;
-  char *tempstr;
+  const char *tempstr;
 
   wdata = data->data;
   if (current_wdata->request->stopable)
@@ -157,7 +160,7 @@ change_filespec (gpointer data)
   gftp_window_data * wdata;
 
   wdata = data;
-  if (!check_status (_("Change Filespec"), wdata, 0, 0, 1))
+  if (!check_status (_("Change Filespec"), wdata, 0, 0, 0, 1))
     return;
 
   MakeEditDialog (_("Change Filespec"), _("Enter the new file specification"),
@@ -172,7 +175,7 @@ dochange_filespec (GtkWidget * widget, gftp_dialog_data * data)
   GList * templist, * filelist;
   gftp_file * tempfle;
   gftp_window_data * wdata;
-  char *edttext;
+  const char *edttext;
   int num;
 
   wdata = data->data;
@@ -267,7 +270,7 @@ site_dialog (gpointer data)
   gftp_window_data * wdata;
 
   wdata = data;
-  if (!check_status (_("Site"), wdata, 0, 0, wdata->request->site != NULL))
+  if (!check_status (_("Site"), wdata, 0, 0, 0, wdata->request->site != NULL))
     return;
 
   MakeEditDialog (_("Site"), _("Enter site-specific command"), NULL, 1, 1,
@@ -279,7 +282,7 @@ static void
 dosite (GtkWidget * widget, gftp_dialog_data * data)
 {
   gftp_window_data * wdata;
-  char *edttext;
+  const char *edttext;
 
   wdata = data->data;
   edttext = gtk_entry_get_text (GTK_ENTRY (data->edit));
@@ -303,7 +306,8 @@ int
 chdir_edit (GtkWidget * widget, gpointer data)
 {
   gftp_window_data * wdata;
-  char *tempstr, *edttxt; 
+  const char *edttxt; 
+  char *tempstr;
 
   wdata = data;
   edttxt = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry));
@@ -313,7 +317,8 @@ chdir_edit (GtkWidget * widget, gpointer data)
       return (0);
     }
 
-  if (!check_status (_("Chdir"), wdata, 0, 0, wdata->request->chdir != NULL))
+  if (!check_status (_("Chdir"), wdata, wdata->request->use_threads, 0, 0, 
+                     wdata->request->chdir != NULL))
     return (FALSE);
 
   if ((tempstr = expand_path (edttxt)) == NULL)
@@ -346,7 +351,8 @@ chdir_dialog (gpointer data)
   int num, ret;
 
   wdata = data;
-  if (!check_status (_("Chdir"), wdata, 1, 0, wdata->request->chdir != NULL))
+  if (!check_status (_("Chdir"), wdata, wdata->request->use_threads, 1, 0, 
+                     wdata->request->chdir != NULL))
     return (0);
 
   filelist = g_list_first (wdata->files);
@@ -423,15 +429,20 @@ do_change_dir_thread (void * data)
   wdata = data;
   wdata->request->user_data = (void *) 0x01;
 
-  sj = sigsetjmp (envir, 1);
-  signal (SIGINT, sig_quit);
-  signal (SIGALRM, sig_quit);
+  if (wdata->request->use_threads)
+    {
+      sj = sigsetjmp (chdirenvir, 1);
+      signal (SIGINT, sig_chdirquit);
+      signal (SIGALRM, sig_chdirquit);
+    }
+  else
+    sj = 0;
 
   success = 0;
   if (sj == 0) 
     {
-      if (connect_timeout > 0)
-        alarm (connect_timeout);
+      if (wdata->request->connect_timeout > 0)
+        alarm (wdata->request->connect_timeout);
       success = gftp_set_directory (wdata->request, wdata->request->directory);
       alarm (0);
     }
@@ -443,22 +454,44 @@ do_change_dir_thread (void * data)
                                         _("Operation canceled\n"));
     }
 
-  signal (SIGINT, SIG_DFL);
-  signal (SIGALRM, SIG_IGN);
+  if (wdata->request->use_threads)
+    {
+      signal (SIGINT, SIG_DFL);
+      signal (SIGALRM, SIG_IGN);
+    }
+
   wdata->request->user_data = NULL;
   wdata->request->stopable = 0;
   return ((void *) success);
 }
 
 
+static RETSIGTYPE
+sig_chdirquit (int signo)
+{
+  signal (signo, sig_chdirquit);
+  siglongjmp (chdirenvir, signo == SIGINT ? 1 : 2);
+}
+
+
 void 
 clearlog (gpointer data)
 {
-  guint pos;
+  guint len;
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+  len = gtk_text_get_length (GTK_TEXT (logwdw));
+  gtk_text_set_point (GTK_TEXT (logwdw), len);
+  gtk_text_backward_delete (GTK_TEXT (logwdw), len);
+#else
+  GtkTextBuffer * textbuf;
+  GtkTextIter iter, iter2;
 
-  pos = gtk_text_get_length (GTK_TEXT (logwdw));
-  gtk_text_set_point (GTK_TEXT (logwdw), pos);
-  gtk_text_backward_delete (GTK_TEXT (logwdw), pos);
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
+  len = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, len - 1);
+  gtk_text_buffer_delete (textbuf, &iter, &iter2);
+#endif
 }
 
 
@@ -469,34 +502,49 @@ viewlog (gpointer data)
   guint textlen;
   ssize_t len;
   int fd;
+#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+  GtkTextBuffer * textbuf;
+  GtkTextIter iter, iter2;
+#endif
 
   tempstr = g_strconcat (tmp_directory, "/gftp-view.XXXXXXXXXX", NULL);
-  if ((fd = mkstemp (tempstr)) == -1)
+  if ((fd = mkstemp (tempstr)) < 0)
     {
       ftp_log (gftp_logging_misc, NULL, _("Error: Cannot open %s: %s\n"),
 	       tempstr, g_strerror (errno));
       g_free (tempstr); 
       return;
     }
+  chmod (tempstr, S_IRUSR | S_IWUSR);
+  unlink (tempstr);
   
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   textlen = gtk_text_get_length (GTK_TEXT (logwdw));
   txt = gtk_editable_get_chars (GTK_EDITABLE (logwdw), 0, -1);
+#else
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
+  textlen = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, textlen - 1);
+  txt = gtk_text_buffer_get_text (textbuf, &iter, &iter2, 0);
+#endif
   pos = txt;
   do 
     {
       if ((len = write (fd, pos, textlen)) == -1)
         { 
-          ftp_log (gftp_logging_misc, NULL, _("Error: Error writing to %s\n"), 
-                   tempstr);
+          ftp_log (gftp_logging_misc, NULL, _("Error: Error writing to %s: %s\n"), tempstr, g_strerror (errno));
           break;
         }
       textlen -= len;
       pos += len;
     } while (textlen > 0);
 
+  lseek (fd, 0, SEEK_SET);
+  view_file (tempstr, fd, 1, 0, 0, 1, NULL, NULL);
   close (fd);
-  view_file (tempstr, 1, 1, 0, NULL, NULL);
   g_free (tempstr);
+  g_free (txt);
 }
 
 
@@ -515,6 +563,7 @@ savelog (gpointer data)
   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (filew));
 
   gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "gftp.log");
+  gtk_window_set_wmclass (GTK_WINDOW(filew), "Save Log", "gFTP");
   gtk_widget_show (filew);
 }
 
@@ -522,9 +571,16 @@ savelog (gpointer data)
 static void
 dosavelog (GtkWidget * widget, GtkFileSelection * fs)
 {
-  char *filename, *txt;
+  const char *filename;
+  char *txt, *pos;
   guint textlen;
+  ssize_t len;
   FILE *fd;
+  int ok;
+#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+  GtkTextBuffer * textbuf;
+  GtkTextIter iter, iter2;
+#endif
 
   filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
   if ((fd = fopen (filename, "w")) == NULL)
@@ -534,14 +590,33 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
       return;
     }
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   textlen = gtk_text_get_length (GTK_TEXT (logwdw));
   txt = gtk_editable_get_chars (GTK_EDITABLE (logwdw), 0, -1);
+#else
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
+  textlen = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, textlen - 1);
+  txt = gtk_text_buffer_get_text (textbuf, &iter, &iter2, 0);
+#endif
 
-  fwrite (txt, 1, textlen, fd);
-  if (ferror (fd))
-    ftp_log (gftp_logging_misc, NULL, _("Error: Error writing to %s: %s\n"),
-             filename, g_strerror (errno));
-  else
+  ok = 1;
+  pos = txt;
+  do
+    {
+      if ((len = write (fileno (fd), pos, textlen)) == -1)
+        {
+          ok = 0;
+          ftp_log (gftp_logging_misc, NULL, _("Error: Error writing to %s: %s\n"), filename, g_strerror (errno));
+          break;
+        }
+
+      textlen -= len;
+      pos += len;
+    } while (textlen > 0);
+
+  if (ok)
     ftp_log (gftp_logging_misc, NULL,
              _("Successfully wrote the log file to %s\n"), filename);
 
@@ -563,12 +638,19 @@ about_dialog (gpointer data)
   GtkWidget * tempwid, * notebook, * box, * label, * view, * vscroll,
             * dialog;
   char *tempstr, *no_license_agreement, *str, buf[255];
+  size_t len;
   FILE * fd;
+#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+  GtkTextBuffer * textbuf;
+  GtkTextIter iter;
+  guint textlen;
+#endif
 
   no_license_agreement = g_strdup_printf (_("Cannot find the license agreement file COPYING. Please make sure it is in either %s or in %s"), BASE_CONF_DIR, SHARE_DIR);
 
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("About gFTP"));
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "about", "gFTP");
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER
 			      (GTK_DIALOG (dialog)->action_area), 5);
@@ -593,7 +675,7 @@ about_dialog (gpointer data)
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
   gtk_widget_show (tempwid);
 
-  tempstr = g_strdup_printf (_("%s\nCopyright (C) 1998-2001 Brian Masney <masneyb@seul.org>\nOfficial Homepage: http://gftp.seul.org/\nLogo by: Aaron Worley <planet_hoth@yahoo.com>\n"),
+  tempstr = g_strdup_printf (_("%s\nCopyright (C) 1998-2002 Brian Masney <masneyb@seul.org>\nOfficial Homepage: http://gftp.seul.org/\nLogo by: Aaron Worley <planet_hoth@yahoo.com>\n"),
 version);
   str = _("Translated by");
   if (strcmp (str, "Translated by") != 0)
@@ -619,18 +701,40 @@ version);
   gtk_box_pack_start (GTK_BOX (box), tempwid, TRUE, TRUE, 0);
   gtk_widget_show (tempwid);
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   view = gtk_text_new (NULL, NULL);
   gtk_text_set_editable (GTK_TEXT (view), FALSE);
   gtk_text_set_word_wrap (GTK_TEXT (view), TRUE);
+
   gtk_table_attach (GTK_TABLE (tempwid), view, 0, 1, 0, 1,
-		    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-		    0, 0);
+                    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
+                    0, 0);
   gtk_widget_show (view);
 
   vscroll = gtk_vscrollbar_new (GTK_TEXT (view)->vadj);
   gtk_table_attach (GTK_TABLE (tempwid), vscroll, 1, 2, 0, 1,
-		    GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
+                    GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
   gtk_widget_show (vscroll);
+#else
+  view = gtk_text_view_new ();
+  gtk_text_view_set_editable (GTK_TEXT_VIEW (view), FALSE);
+  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view), GTK_WRAP_WORD);
+
+  vscroll = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (vscroll),
+                                 GTK_POLICY_AUTOMATIC,
+                                 GTK_POLICY_AUTOMATIC);
+
+  gtk_container_add (GTK_CONTAINER (vscroll), view);
+  gtk_widget_show (view);
+
+  gtk_table_attach (GTK_TABLE (tempwid), vscroll, 0, 1, 0, 1,
+                    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND | GTK_SHRINK,
+                    0, 0);
+  gtk_widget_show (vscroll);
+
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+#endif
 
   label = gtk_label_new (_("License Agreement"));
   gtk_widget_show (label);
@@ -647,7 +751,7 @@ version);
   gtk_widget_grab_default (tempwid);
   gtk_widget_show (tempwid);
 
-  tempstr = g_strconcat ("/usr/doc/copyright/GPL", NULL);
+  tempstr = g_strconcat ("/usr/share/common-licenses/GPL", NULL);
   if (access (tempstr, F_OK) != 0)
     {
       g_free (tempstr);
@@ -658,18 +762,30 @@ version);
           tempstr = expand_path (BASE_CONF_DIR "/COPYING");
 	  if (access (tempstr, F_OK) != 0)
 	    {
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
 	      gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL,
-			       no_license_agreement,
-			       strlen (no_license_agreement));
+			       no_license_agreement, -1);
+#else
+              textlen = gtk_text_buffer_get_char_count (textbuf);
+              gtk_text_buffer_get_iter_at_offset (textbuf, &iter, textlen - 1);
+              gtk_text_buffer_insert (textbuf, &iter, no_license_agreement, -1);
+#endif
 	      gtk_widget_show (dialog);
 	      return;
 	    }
 	}
     }
+
   if ((fd = fopen (tempstr, "r")) == NULL)
     {
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
       gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL,
-		       no_license_agreement, strlen (no_license_agreement));
+		       no_license_agreement, -1);
+#else
+      textlen = gtk_text_buffer_get_char_count (textbuf);
+      gtk_text_buffer_get_iter_at_offset (textbuf, &iter, textlen - 1);
+      gtk_text_buffer_insert (textbuf, &iter, no_license_agreement, -1);
+#endif
       gtk_widget_show (dialog);
       g_free (tempstr);
       return;
@@ -677,8 +793,17 @@ version);
   g_free (tempstr);
 
   memset (buf, 0, sizeof (buf));
-  while (fread (buf, 1, sizeof (buf) - 1, fd))
-    gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL, buf, strlen (buf));
+  while ((len = fread (buf, 1, sizeof (buf) - 1, fd)))
+    {
+      buf[len] = '\0';
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+      gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL, buf, -1);
+#else
+      textlen = gtk_text_buffer_get_char_count (textbuf);
+      gtk_text_buffer_get_iter_at_offset (textbuf, &iter, textlen - 1);
+      gtk_text_buffer_insert (textbuf, &iter, buf, -1);
+#endif
+    }
   fclose (fd);
   gtk_widget_show (dialog);
   g_free (no_license_agreement);
@@ -693,7 +818,7 @@ compare_windows (gpointer data)
   GList * curlist, * otherlist;
   int row;
 
-  if (!check_status (_("Compare Windows"), &window2, 0, 0, 1))
+  if (!check_status (_("Compare Windows"), &window2, 1, 0, 0, 1))
     return;
 
   deselectall (&window1);
