@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  misc-gtk.c - misc stuff for the gtk+ 1.2 port of gftp                    */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -53,11 +53,20 @@ remove_files_window (gftp_window_data * wdata)
 void
 ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
 {
+  guint len, delsize;
   char tempstr[512];
   gftp_log * newlog;
-  GdkColor fore;
   va_list argp;
   int upd;
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+  gftp_color * color;
+  GdkColor fore;
+#else
+  GtkTextBuffer * textbuf;
+  GtkTextIter iter, iter2;
+  const char *descr;
+  guint oldlen;
+#endif
 
   if (ptr == (void *) 0x1)
     {
@@ -81,17 +90,6 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
       return;
     }
 
-  upd = GTK_TEXT (logwdw)->vadj->upper - GTK_TEXT (logwdw)->vadj->page_size ==
-    GTK_TEXT (logwdw)->vadj->value;
-
-  memset (&fore, 0, sizeof (fore));
-  if (level == gftp_logging_send)
-    fore.green = 0x8600;
-  else if (level == gftp_logging_recv)
-    fore.blue = 0xffff;
-  else
-    fore.red = 0xffff;
-
   va_start (argp, string);
   g_vsnprintf (tempstr, sizeof (tempstr), string, argp);
   va_end (argp);
@@ -101,8 +99,6 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
   else if (strncmp (tempstr, "ACCT", 4) == 0)
     strncpy (tempstr, "ACCT xxxx\r\n", sizeof (tempstr));
 
-  gtk_text_freeze (GTK_TEXT (logwdw));
-
   if (logfd != NULL)
     {
       if (fwrite (tempstr, strlen (tempstr), 1, logfd) != 1)
@@ -114,17 +110,85 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
         fflush (logfd);
     }
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+  switch (level)
+    {
+      case gftp_logging_send:
+        color = &send_color;
+        break;
+      case gftp_logging_recv:
+        color = &recv_color;
+        break;
+      case gftp_logging_error:
+        color = &error_color;
+        break;
+      default:
+        color = &misc_color;
+        break;
+    }
+
+  memset (&fore, 0, sizeof (fore));
+  fore.red = color->red;
+  fore.green = color->green;
+  fore.blue = color->blue;
+
+  upd = logwdw_vadj->upper - logwdw_vadj->page_size == logwdw_vadj->value;
+
+  gtk_text_freeze (GTK_TEXT (logwdw));
   gtk_text_insert (GTK_TEXT (logwdw), NULL, &fore, NULL, tempstr, -1);
 
-  gtk_text_set_point (GTK_TEXT (logwdw), 
-                      gtk_text_get_length (GTK_TEXT (logwdw)));
+  len = gtk_text_get_length (GTK_TEXT (logwdw));
+  if (max_log_window_size > 0 && len > max_log_window_size)
+    {
+      delsize = len - max_log_window_size;
+      gtk_text_set_point (GTK_TEXT (logwdw), delsize);
+      gtk_text_backward_delete (GTK_TEXT (logwdw), delsize);
+      len = max_log_window_size;
+    }
+  gtk_text_set_point (GTK_TEXT (logwdw), len);
 
   gtk_text_thaw (GTK_TEXT (logwdw));
+#else
+  switch (level)
+    {
+      case gftp_logging_send:
+        descr = "send";
+        break;
+      case gftp_logging_recv:
+        descr = "recv";
+        break;
+      case gftp_logging_error:
+        descr = "error";
+        break;
+      default:
+        descr = "misc";
+        break;
+    }
+
+  upd = logwdw_vadj->upper - logwdw_vadj->page_size == logwdw_vadj->value;
+
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
+  oldlen = len = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, len - 1);
+  gtk_text_buffer_insert (textbuf, &iter, tempstr, -1);
+
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, oldlen - 1);
+  len = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, len - 1);
+  gtk_text_buffer_apply_tag_by_name (textbuf, descr, &iter, &iter2);
+
+  if (max_log_window_size > 0 && len > max_log_window_size)
+    {
+      delsize = len - max_log_window_size;
+      gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+      gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, delsize);
+      gtk_text_buffer_delete (textbuf, &iter, &iter2);
+    }
+
+#endif
 
   if (upd)
-    gtk_adjustment_set_value (GTK_TEXT (logwdw)->vadj,
-			      GTK_TEXT (logwdw)->vadj->upper -
-                              GTK_TEXT (logwdw)->vadj->page_size);
+    gtk_adjustment_set_value (logwdw_vadj, logwdw_vadj->upper);
 
   if (ptr == NULL)
     fix_display ();
@@ -134,7 +198,7 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
 void
 refresh (gftp_window_data * wdata)
 {
-  if (!check_status (_("Refresh"), wdata, 0, 0, 1))
+  if (!check_status (_("Refresh"), wdata, 0, 0, 0, 1))
     return;
   gtk_clist_freeze (GTK_CLIST (wdata->listbox));
   remove_files_window (wdata);
@@ -293,8 +357,7 @@ toolbar_pixmap (GtkWidget * widget, char *filename)
   if (filename == NULL || *filename == '\0')
     return (NULL);
 
-  if ((graphic = g_hash_table_lookup (graphic_hash_table, filename)) == NULL)
-    graphic = open_xpm (widget, filename);
+  graphic = open_xpm (widget, filename);
 
   if (graphic == NULL)
     return (NULL);
@@ -314,6 +377,9 @@ open_xpm (GtkWidget * widget, char *filename)
   GtkStyle *style;
   char *exfile;
 
+  if ((graphic = g_hash_table_lookup (graphic_hash_table, filename)) != NULL)
+    return (graphic);
+
   style = gtk_widget_get_style (widget);
 
   if ((exfile = get_xpm_path (filename, 0)) == NULL)
@@ -348,8 +414,14 @@ gftp_free_pixmap (char *filename)
   if ((graphic = g_hash_table_lookup (graphic_hash_table, filename)) == NULL)
     return;
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gdk_pixmap_unref (graphic->pixmap);
   gdk_bitmap_unref (graphic->bitmap);
+#else
+  gdk_drawable_unref (graphic->pixmap);
+  gdk_drawable_unref (graphic->bitmap);
+#endif
+
   g_hash_table_remove (graphic_hash_table, filename);
   g_free (graphic->filename);
   g_free (graphic);
@@ -385,9 +457,12 @@ gftp_get_pixmap (GtkWidget * widget, char *filename, GdkPixmap ** pix,
 
 
 int
-check_status (char *name, gftp_window_data *wdata, int only_one,
-	      int at_least_one, int func)
+check_status (char *name, gftp_window_data *wdata, int check_other_stop,
+              int only_one, int at_least_one, int func)
 {
+  gftp_window_data * owdata;
+
+  owdata = wdata == &window1 ? &window2 : &window1;
 
   if (wdata->request->stopable)
     {
@@ -397,6 +472,14 @@ check_status (char *name, gftp_window_data *wdata, int only_one,
       return (0);
     }
 
+  if (check_other_stop && owdata->request->stopable)
+    {
+      ftp_log (gftp_logging_misc, NULL,
+	       _("%s: Please hit the stop button first to do anything else\n"),
+	       name);
+      return (0);
+    }
+
   if (!GFTP_IS_CONNECTED (wdata->request))
     {
       ftp_log (gftp_logging_misc, NULL,
@@ -495,7 +578,7 @@ get_next_selection (GList * selection, GList ** list, int *curnum)
 
 void
 add_history (GtkWidget * widget, GList ** history, unsigned int *histlen, 
-             char *str)
+             const char *str)
 {
   GList *node, *delnode;
   char *tempstr;
@@ -665,6 +748,7 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int edit_shown,
   d->dialog = cancel_d->dialog = gtk_dialog_new ();
   gtk_grab_add (d->dialog);
   gtk_window_set_title (GTK_WINDOW (d->dialog), diagtxt);
+  gtk_window_set_wmclass (GTK_WINDOW(d->dialog), "edit", "gFTP");
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (d->dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER
 			      (GTK_DIALOG (d->dialog)->action_area), 5);
@@ -764,6 +848,7 @@ MakeYesNoDialog (char *diagtxt, char *infotxt, int erase, int num, ...)
   dialog = gtk_dialog_new ();
   gtk_grab_add (dialog);
   gtk_window_set_title (GTK_WINDOW (dialog), diagtxt);
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "yndiag", "gFTP");
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 
                               5);
@@ -842,13 +927,14 @@ update_directory_download_progress (gftp_transfer * transfer)
 
   if (dialog == NULL)
     {
-      dialog = gtk_window_new (GTK_WINDOW_DIALOG);
+      dialog = gtk_window_new (GTK_WINDOW_POPUP);
       gtk_grab_add (dialog);
       gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
                           GTK_SIGNAL_FUNC (delete_event), NULL);
       gtk_window_set_title (GTK_WINDOW (dialog),
 			    _("Getting directory listings"));
       gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+      gtk_window_set_wmclass (GTK_WINDOW(dialog), "dirlist", "gFTP");
 
       vbox = gtk_vbox_new (FALSE, 5);
       gtk_container_border_width (GTK_CONTAINER (vbox), 10);
@@ -859,6 +945,7 @@ update_directory_download_progress (gftp_transfer * transfer)
       gtk_box_pack_start (GTK_BOX (vbox), textwid, TRUE, TRUE, 0);
       gtk_widget_show (textwid);
 
+      /* FIXME - gtk 2.0 has a better API */
       statuswid = gtk_progress_bar_new ();
       gtk_progress_set_activity_mode (GTK_PROGRESS (statuswid), 1);
       gtk_progress_bar_set_activity_step (GTK_PROGRESS_BAR (statuswid), 3);
@@ -968,8 +1055,7 @@ display_cached_logs (void)
   pthread_mutex_unlock (&log_mutex);
 }
 
-
-#ifndef HAVE_GETADDRINFO
+#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
 
 static pthread_mutex_t netfunclock = PTHREAD_MUTEX_INITIALIZER;
 
