@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  menu-items.c - menu callbacks                                            */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -40,21 +40,22 @@ static char *get_proxy_config 			( void );
 
 static GtkWidget * proxy_text, * proxy_list, * new_proxy_domain, * network1,
                  * network2, * network3, * network4, * netmask1, * netmask2, 
-                 * netmask3, * netmask4, * domain_active, * proxy_combo;
+                 * netmask3, * netmask4, * domain_active, * proxy_combo,
+                 * def_proto_combo;
 static GList * new_proxy_hosts;
 static char *custom_proxy;
-static int proxy_num;
 
 void
 options_dialog (gpointer data)
 {
   GtkWidget * dialog, * tempwid, * notebook, * table, * box;
   char tempstr[20], *pos, *endpos, *oldstr;
-  int i, tbl_col, tbl_num, proxy_num;
-  GList * proxy_list;
+  int i, tbl_col, tbl_num, combo_num;
+  GList * combo_list;
 
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Options"));
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "options", "gFTP");
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area),
                               5);
@@ -72,6 +73,9 @@ options_dialog (gpointer data)
   table = box = NULL;
   for (i=0; config_file_vars[i].key != NULL; i++)
     {
+      if (!config_file_vars[i].shown)
+        continue;
+
       switch (config_file_vars[i].type)
         {
           case CONFIG_NOTEBOOK:
@@ -101,18 +105,32 @@ options_dialog (gpointer data)
                                        tbl_num - 1, tbl_num);
             gtk_widget_show (tempwid);
 
-            proxy_combo = gtk_combo_new ();
-            gtk_table_attach_defaults (GTK_TABLE (table), proxy_combo, 1, 2,
+            tempwid = gtk_combo_new ();
+            gtk_table_attach_defaults (GTK_TABLE (table), tempwid, 1, 2,
                                        tbl_num - 1, tbl_num);
-            gtk_widget_show (proxy_combo);
-            config_file_vars[i].widget = proxy_combo;
+            gtk_widget_show (tempwid);
+            config_file_vars[i].widget = tempwid;
+
+            /* We only have Default Protocol and the Proxy type as the two
+               combo types. If I add more later on, I'll work on a better
+               interface for all this stuff */
+            if (strcmp (config_file_vars[i].comment, "DP") == 0)
+              def_proto_combo = tempwid;
+            else
+              proxy_combo = tempwid;
 
             tbl_num++;
             break;
           case CONFIG_TEXT:
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
             proxy_text = gtk_text_new (NULL, NULL);
-            gtk_widget_set_usize (proxy_text, -2, 125);
+            gtk_widget_set_usize (proxy_text, -1, 125);
             gtk_text_set_editable (GTK_TEXT (proxy_text), TRUE);
+#else
+            proxy_text = gtk_text_view_new ();
+            gtk_widget_set_size_request (proxy_text, -1, 125);
+            gtk_text_view_set_editable (GTK_TEXT_VIEW (proxy_text), TRUE);
+#endif
             gtk_table_attach_defaults (GTK_TABLE (table), proxy_text, 0, 2, 
                                        tbl_num - 1, tbl_num);
             gtk_widget_show (proxy_text);
@@ -121,7 +139,9 @@ options_dialog (gpointer data)
             tbl_num++;
             break;
           case CONFIG_CHARTEXT:
+          case CONFIG_CHARPASS:
           case CONFIG_INTTEXT:
+          case CONFIG_UINTTEXT:
           case CONFIG_FLOATTEXT:
             gtk_table_resize (GTK_TABLE (table), tbl_num, 2);
 
@@ -138,6 +158,7 @@ options_dialog (gpointer data)
             switch (config_file_vars[i].type)
               {
                 case CONFIG_INTTEXT:
+                case CONFIG_UINTTEXT:
                   g_snprintf (tempstr, sizeof (tempstr), "%d",
                               *(int *) config_file_vars[i].var);
                   gtk_entry_set_text (GTK_ENTRY (tempwid), tempstr);
@@ -151,6 +172,11 @@ options_dialog (gpointer data)
                   gtk_entry_set_text (GTK_ENTRY (tempwid),
                                       *(char **) config_file_vars[i].var);
                   break;
+                case CONFIG_CHARPASS:
+                  gtk_entry_set_text (GTK_ENTRY (tempwid),
+                                      *(char **) config_file_vars[i].var);
+                  gtk_entry_set_visibility (GTK_ENTRY (tempwid), 0);
+                  break;
               }
             gtk_widget_show (tempwid);
             config_file_vars[i].widget = tempwid;
@@ -191,17 +217,34 @@ options_dialog (gpointer data)
         }
     }
 
-  proxy_list = NULL;
+  combo_num = 0;
+  combo_list = NULL;
+  for (i = 0; gftp_protocols[i].name != NULL; i++)
+    {
+      if (strcmp (default_protocol, gftp_protocols[i].name) == 0)
+        combo_num = i;
+      tempwid = gtk_list_item_new_with_label (gftp_protocols[i].name);
+      gtk_widget_show (tempwid);
+      combo_list = g_list_append (combo_list, tempwid);
+    }
+  gtk_list_prepend_items (GTK_LIST (GTK_COMBO (def_proto_combo)->list), 
+                          combo_list); 
+  gtk_list_select_item (GTK_LIST (GTK_COMBO (def_proto_combo)->list), 
+                        combo_num);
+
+  combo_list = NULL;
   for (i = 0; proxy_type[i].key != NULL; i++)
     {
       tempwid = gtk_list_item_new_with_label (_(proxy_type[i].key));
       gtk_widget_show (tempwid);
-      proxy_list = g_list_append (proxy_list, tempwid);
+      combo_list = g_list_append (combo_list, tempwid);
     }
-  gtk_list_prepend_items (GTK_LIST (GTK_COMBO (proxy_combo)->list), proxy_list);
+  gtk_list_prepend_items (GTK_LIST (GTK_COMBO (proxy_combo)->list), combo_list);
+  combo_list = NULL;
+
   custom_proxy = g_malloc0 (1);
   if (proxy_config == NULL || *proxy_config == '\0')
-    proxy_num= 0;
+    combo_num = 0;
   else
     {
       pos = proxy_config;
@@ -221,16 +264,16 @@ options_dialog (gpointer data)
           g_free (oldstr);
         }
 
-      for (proxy_num = 1; proxy_num < GFTP_CUSTOM_PROXY_NUM; proxy_num++)
+      for (combo_num = 1; combo_num < GFTP_CUSTOM_PROXY_NUM; combo_num++)
         {
-          if (strcmp (proxy_type[proxy_num].description, custom_proxy) == 0)
+          if (strcmp (proxy_type[combo_num].description, custom_proxy) == 0)
             break;
         }
     }
 
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (proxy_combo)->list),
                       "select_child", GTK_SIGNAL_FUNC (proxy_toggle), NULL);
-  gtk_list_select_item (GTK_LIST (GTK_COMBO (proxy_combo)->list), proxy_num);
+  gtk_list_select_item (GTK_LIST (GTK_COMBO (proxy_combo)->list), combo_num);
 
   make_proxy_hosts_tab (notebook);
 
@@ -402,6 +445,7 @@ add_proxy_host (GtkWidget * widget, gpointer data)
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog),
 			hosts ? _("Edit Host") : _("Add Host"));
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "hostinfo", "Gftp");
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER
 			      (GTK_DIALOG (dialog)->action_area), 5);
@@ -454,22 +498,46 @@ add_proxy_host (GtkWidget * widget, gpointer data)
   gtk_widget_show (box);
 
   network1 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (network1, 28, -1);
+#else
+  gtk_widget_set_size_request (network1, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), network1, TRUE, TRUE, 0);
   gtk_widget_show (network1);
 
   network2 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (network2, 28, -1);
+#else
+  gtk_widget_set_size_request (network2, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), network2, TRUE, TRUE, 0);
   gtk_widget_show (network2);
 
   network3 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (network3, 28, -1);
+#else
+  gtk_widget_set_size_request (network3, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), network3, TRUE, TRUE, 0);
   gtk_widget_show (network3);
 
   network4 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (network4, 28, -1);
+#else
+  gtk_widget_set_size_request (network4, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), network4, TRUE, TRUE, 0);
   gtk_widget_show (network4);
 
@@ -483,22 +551,46 @@ add_proxy_host (GtkWidget * widget, gpointer data)
   gtk_widget_show (box);
 
   netmask1 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (netmask1, 28, -1);
+#else
+  gtk_widget_set_size_request (netmask1, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), netmask1, TRUE, TRUE, 0);
   gtk_widget_show (netmask1);
 
   netmask2 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (netmask2, 28, -1);
+#else
+  gtk_widget_set_size_request (netmask2, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), netmask2, TRUE, TRUE, 0);
   gtk_widget_show (netmask2);
 
   netmask3 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (netmask3, 28, -1);
+#else
+  gtk_widget_set_size_request (netmask3, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), netmask3, TRUE, TRUE, 0);
   gtk_widget_show (netmask3);
 
   netmask4 = gtk_entry_new ();
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_widget_set_usize (netmask4, 28, -1);
+#else
+  gtk_widget_set_size_request (netmask4, 28, -1);
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), netmask4, TRUE, TRUE, 0);
   gtk_widget_show (netmask4);
 
@@ -615,8 +707,8 @@ static void
 add_ok (GtkWidget * widget, gpointer data)
 {
   gftp_proxy_hosts *hosts;
+  const char *edttxt;
   GList *templist;
-  char *edttxt;
   int num;
 
   if ((templist = data) == NULL)
@@ -695,27 +787,47 @@ delete_proxy_host (GtkWidget * widget, gpointer data)
 static void
 proxy_toggle (GtkList * list, GtkWidget * child, gpointer data)
 {
+  int proxy_num;
+  char *str;
+
+#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+  GtkTextIter iter, iter2;
+  GtkTextBuffer * textbuf;
+  guint len;
+#endif
+
+  proxy_num = gtk_list_child_position (list, child);
+  if (proxy_num == GFTP_CUSTOM_PROXY_NUM)
+    str = custom_proxy;
+  else
+    str = proxy_type[proxy_num].description;
+
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   gtk_text_set_point (GTK_TEXT (proxy_text), 0);
   gtk_text_forward_delete (GTK_TEXT (proxy_text),
 			   gtk_text_get_length (GTK_TEXT (proxy_text)));
 
-  proxy_num = gtk_list_child_position (list, child);
-  if (proxy_num == GFTP_CUSTOM_PROXY_NUM)
-    gtk_text_insert (GTK_TEXT (proxy_text), NULL, NULL, NULL,
-		     custom_proxy, strlen (custom_proxy));
-  else
-    gtk_text_insert (GTK_TEXT (proxy_text), NULL, NULL, NULL,
-		     proxy_type[proxy_num].description,
-		     strlen (proxy_type[proxy_num].description));
+  gtk_text_insert (GTK_TEXT (proxy_text), NULL, NULL, NULL, str, -1);
+#else
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (proxy_text));
+  len = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, len - 1);
+  gtk_text_buffer_delete (textbuf, &iter, &iter2);
+
+  len = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, len - 1);
+  gtk_text_buffer_insert (textbuf, &iter, str, -1);
+#endif
 }
 
 
 static void
 apply_changes (GtkWidget * widget, gpointer data)
 {
+  const char *tempstr;
+  int num, found, i;
   GList *templist;
-  char *tempstr;
-  int num;
 
   for (num = 0; config_file_vars[num].var != NULL; num++)
     {
@@ -728,6 +840,7 @@ apply_changes (GtkWidget * widget, gpointer data)
                       GTK_TOGGLE_BUTTON (config_file_vars[num].widget)->active;
                 break;
               case CONFIG_INTTEXT:
+              case CONFIG_UINTTEXT:
                 tempstr = gtk_entry_get_text ( 
                                GTK_ENTRY (config_file_vars[num].widget));
                 *(int *) config_file_vars[num].var = strtol (tempstr, NULL, 10);
@@ -738,6 +851,7 @@ apply_changes (GtkWidget * widget, gpointer data)
                 *(double *) config_file_vars[num].var = strtod (tempstr, NULL);
                 break;
               case CONFIG_CHARTEXT:
+              case CONFIG_CHARPASS:
                 tempstr = gtk_entry_get_text ( 
                                GTK_ENTRY (config_file_vars[num].widget));
                 g_free (*(char **) config_file_vars[num].var);
@@ -749,17 +863,29 @@ apply_changes (GtkWidget * widget, gpointer data)
         }
     }
 
+  tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (def_proto_combo)->entry));
+  found = 0;
+  for (i = 0; gftp_protocols[i].name; i++)
+    {
+      if (strcmp (gftp_protocols[i].name, tempstr) == 0)
+        {
+          found = 1;
+          break;
+        }
+    }
+
+  if (found)
+    {
+      g_free (default_protocol);
+      default_protocol = g_strconcat (tempstr, NULL);
+    }
+
   templist = proxy_hosts;
   proxy_hosts = new_proxy_hosts;
   new_proxy_hosts = templist;
   clean_old_changes (NULL, NULL);
   proxy_config = get_proxy_config ();
 
-  GFTP_SET_TRANSFER_TYPE (window1.request,
-              passive_transfer ? gftp_transfer_passive : gftp_transfer_active);
-  GFTP_SET_TRANSFER_TYPE (window2.request,
-              passive_transfer ? gftp_transfer_passive : gftp_transfer_active);
-
   gftp_write_config_file ();
 }
 
@@ -787,13 +913,20 @@ clean_old_changes (GtkWidget * widget, gpointer data)
 static char *
 get_proxy_config (void)
 {
-  char *newstr, *oldstr, *pos, *endpos, tempchar, tmp[128];
-  wchar_t *wpos, tempwchar;
-  unsigned int len;
+  char *newstr, *oldstr, *pos, *endpos, *textstr;
+  guint len;
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+  char tmp[128];
+#else
+  GtkTextBuffer * textbuf;
+  GtkTextIter iter, iter2;
+#endif
 
+  textstr = NULL;
   newstr = g_malloc (1);
   *newstr = '\0';
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   /*
      GTK_TEXT uses wchar_t instead of char in environment of multibyte encoding
      locale (ex Japanese),  so we must convert from wide character 
@@ -806,19 +939,26 @@ get_proxy_config (void)
       pos = tmp;
     }
   else
-    pos = (char *) GTK_TEXT (proxy_text)->text.ch;
+    {
+      oldstr = (char *) GTK_TEXT (proxy_text)->text.ch; 
+      len = gtk_text_get_length (GTK_TEXT (proxy_text));
+      textstr = g_malloc (len + 1);
+      strncpy (textstr, oldstr, len);
+      textstr[len] = '\0';
+      pos = textstr;
+    }
+#else
+  textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (proxy_text));
+  len = gtk_text_buffer_get_char_count (textbuf);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+  gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, len - 1);
+  pos = textstr = gtk_text_buffer_get_text (textbuf, &iter, &iter2, 0);
+#endif
 
-  len = gtk_text_get_length (GTK_TEXT (proxy_text));
-  tempchar = pos[len];
-  pos[len] = '\0';
   do
     {
       if ((endpos = strchr (pos, '\n')) != NULL)
-        {
-          if (*(endpos - 1) == '\r')
-            *(endpos - 1) = '\0';
-          *endpos = '\0';
-        }
+        *endpos = '\0';
       oldstr = newstr;
       if (endpos != NULL)
         newstr = g_strconcat (newstr, pos, "%n", NULL);
@@ -833,17 +973,13 @@ get_proxy_config (void)
     }
   while (endpos != NULL);
 
+#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
   if (GTK_TEXT (proxy_text)->use_wchar)
-    {
-      wpos = (wchar_t *) GTK_TEXT (proxy_text)->text.wc;
-      mbtowc (&tempwchar, &tempchar, sizeof (tempwchar));
-      wpos[len] = tempwchar;
-    }
-  else
-    {
-      pos = (char *) GTK_TEXT (proxy_text)->text.ch;
-      pos[len] = tempchar;
-    }
+    g_free (textstr);
+#else
+  g_free (textstr);
+#endif
+
   return (newstr);
 }
 
