@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-text.c - text port of gftp                                          */
-/*  Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -21,8 +21,7 @@
 
 static gftp_request * gftp_text_locreq = NULL;
 static gftp_request * gftp_text_remreq = NULL;
-static volatile int cancel = 0, 
-                    queuefiles = 0;
+static volatile int cancel = 0;
 
 struct _gftp_text_methods gftp_text_methods[] =
        {{N_("open"), 	gftp_text_open, 	&gftp_text_remreq},
@@ -64,18 +63,29 @@ struct _gftp_text_methods gftp_text_methods[] =
 int
 main (int argc, char **argv)
 {
-  char tempstr[512], *pos, *stpos;
   gftp_request * request;
+  char *pos, *stpos;
   int i;
+#ifdef HAVE_LIBREADLINE
+  char *tempstr, prompt[20];
+#else
+  char tempstr[512];
+#endif
 
 #ifdef HAVE_GETTEXT
   setlocale (LC_ALL, "");
   bindtextdomain ("gftp", LOCALE_DIR);
   textdomain ("gftp");
 #endif
- 
+
+  signal (SIGCHLD, sig_child);
   signal (SIGPIPE, SIG_IGN); 
+
   gftp_read_config_file (0);
+  /* SSH doesn't support reading the password with askpass via the command 
+     line */
+  ssh_use_askpass = sshv2_use_sftp_subsys = 0;
+
   if (gftp_parse_command_line (&argc, &argv) != 0)
     exit (0);
 
@@ -89,7 +99,7 @@ main (int argc, char **argv)
     gftp_set_directory (gftp_text_locreq, startup_directory);
   gftp_connect (gftp_text_locreq);
 
-  gftp_text_log (gftp_logging_misc, NULL, "%s, Copyright (C) 1998-2001 Brian Masney <", version);
+  gftp_text_log (gftp_logging_misc, NULL, "%s, Copyright (C) 1998-2002 Brian Masney <", version);
   gftp_text_log (gftp_logging_recv, NULL, "masneyb@seul.org");
   gftp_text_log (gftp_logging_misc, NULL, _(">.\nIf you have any questions, comments, or suggestions about this program, please feel free to email them to me. You can always find out the latest news about gFTP from my website at http://gftp.seul.org/\n"));
   gftp_text_log (gftp_logging_misc, NULL, "\n");
@@ -111,8 +121,13 @@ main (int argc, char **argv)
   else if (argc == 2)
     gftp_text_open (gftp_text_remreq, argv[1], NULL);
 
+#ifdef HAVE_LIBREADLINE
+  g_snprintf (prompt, sizeof (prompt), "%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
+  while ((tempstr = readline (prompt)))
+#else
   printf ("%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
   while (fgets (tempstr, sizeof (tempstr), stdin) != NULL)
+#endif
     {
       if (tempstr[strlen (tempstr) - 1] == '\n')
         tempstr[strlen (tempstr) - 1] = '\0';
@@ -123,10 +138,16 @@ main (int argc, char **argv)
 
       if (*stpos == '\0')
         {
+#ifndef HAVE_LIBREADLINE
           printf ("%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
+#endif
           continue;
         }
 
+#ifdef HAVE_LIBREADLINE
+      add_history (tempstr);
+#endif
+
       if ((pos = strchr (stpos, ' ')) != NULL)
         *pos = '\0';
 
@@ -155,9 +176,15 @@ main (int argc, char **argv)
         gftp_text_log (gftp_logging_error, NULL, 
                        _("Error: Command not recognized\n"));
 
+#ifdef HAVE_LIBREADLINE
+     free (tempstr);
+#else
      printf ("%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
+#endif
     }
 
+  gftp_clear_cache_files ();
+  fclose (logfd);
   return (0);
 }
 
@@ -260,10 +287,18 @@ gftp_text_open (gftp_request * request, char *command, gpointer *data)
     {
       if ((pos = gftp_text_ask_question ("Username [anonymous]", 1, tempstr, 
                                          sizeof (tempstr))) != NULL)
-        gftp_set_username (request, pos);
+        {
+          gftp_set_username (request, pos);
+          if (request->password)
+            {
+              g_free (request->password);
+              request->password = NULL;
+            }
+        }
     }
 
-  if (strcmp (GFTP_GET_USERNAME (request), "anonymous") != 0)
+  if (strcmp (GFTP_GET_USERNAME (request), "anonymous") != 0 && 
+      (request->password == NULL || *request->password == '\0'))
     {
       if ((pos = gftp_text_ask_question ("Password", 0, tempstr, 
                                          sizeof (tempstr))) == NULL)
@@ -290,7 +325,7 @@ gftp_text_about (gftp_request * request, char *command, gpointer *data)
   char *str;
 
   gftp_text_log (gftp_logging_misc, NULL,
-      "%s. Copyright (C) 1998-2001 Brian Masney <masneyb@seul.org>\n", version);
+      "%s. Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>\n", version);
 
   str = _("Translated by");
   if (strcmp (str, "Translated by") != 0)
@@ -302,7 +337,6 @@ gftp_text_about (gftp_request * request, char *command, gpointer *data)
 int
 gftp_text_quit (gftp_request * request, char *command, gpointer *data)
 {
-  gftp_write_config_file ();
   gftp_clear_cache_files ();
   fclose (logfd);
   return (0);
@@ -461,7 +495,7 @@ gftp_text_chmod (gftp_request * request, char *command, gpointer *data)
     }
   else
     {
-      gftp_chmod (request, command, strtol (pos, NULL, 10));
+      gftp_chmod (request, pos, strtol (command, NULL, 10));
     }
   return (1);
 }
@@ -496,7 +530,7 @@ gftp_text_ls (gftp_request * request, char *command, gpointer *data)
 
       if (*fle.attribs == 'd')
         color = COLOR_BLUE;
-      else if (*fle.attribs == ';')
+      else if (*fle.attribs == 'l')
         color = COLOR_WHITE;
       else if (strchr (fle.attribs, 'x') != NULL)
         color = COLOR_GREEN;
@@ -576,8 +610,12 @@ gftp_text_mget_file (gftp_request * request, char *command, gpointer *data)
 
   /* FIXME - ask whether to resume/skip/overwrite */
   if (gftp_list_files (transfer->fromreq) != 0)
-    return (1);
-  fle = g_malloc (sizeof (*fle));
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
+  fle = g_malloc0 (sizeof (*fle));
   while (gftp_get_next_file (transfer->fromreq, command, fle) > 0)
     {
       if (strcmp (fle->file, ".") == 0 || strcmp (fle->file, "..") == 0)
@@ -591,13 +629,30 @@ gftp_text_mget_file (gftp_request * request, char *command, gpointer *data)
   g_free (fle);
   gftp_end_transfer (transfer->fromreq);
 
+  if (transfer->files == NULL)
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
+
   if (gftp_get_all_subdirs (transfer, NULL) != 0)
-    return (1);
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
 
   if (transfer->files == NULL)
-    return (1);
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
 
   gftp_text_transfer_files (transfer);
+  transfer->fromreq = transfer->toreq = NULL;
+  free_tdata (transfer);
   return (1);
 }
 
@@ -627,7 +682,11 @@ gftp_text_mput_file (gftp_request * request, char *command, gpointer *data)
   transfer->transfer_direction = GFTP_DIRECTION_UPLOAD;
 
   if (gftp_list_files (transfer->fromreq) != 0)
-    return (1);
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
   fle = g_malloc (sizeof (*fle));
   while (gftp_get_next_file (transfer->fromreq, command, fle) > 0)
     {
@@ -642,13 +701,30 @@ gftp_text_mput_file (gftp_request * request, char *command, gpointer *data)
   g_free (fle);
   gftp_end_transfer (transfer->fromreq);
 
+  if (transfer->files == NULL)
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
+
   if (gftp_get_all_subdirs (transfer, NULL) != 0)
-    return (1);
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
 
   if (transfer->files == NULL)
-    return (1);
+    {
+      transfer->fromreq = transfer->toreq = NULL;
+      free_tdata (transfer);
+      return (1);
+    }
 
   gftp_text_transfer_files (transfer);
+  transfer->fromreq = transfer->toreq = NULL;
+  free_tdata (transfer);
   return (1);
 }
 
@@ -657,7 +733,7 @@ int
 gftp_text_transfer_files (gftp_transfer * transfer)
 {
   char *tempstr, buf[8192], *progress = "|/-\\";
-  struct timeval starttime, lasttime, updatetime;
+  struct timeval updatetime;
   long fromsize, total;
   gftp_file * curfle;
   int i, j, sw, tot;
@@ -684,14 +760,16 @@ gftp_text_transfer_files (gftp_transfer * transfer)
                          maxkbs);
         }
 
-      fromsize = gftp_transfer_file (transfer->fromreq, curfle->file,
-                                     curfle->startsize, transfer->toreq, 
-                                     curfle->destfile, curfle->startsize);
+      transfer->curtrans = curfle->startsize;
+      fromsize = gftp_transfer_file (transfer->fromreq, curfle->file, NULL,
+                                     curfle->startsize, transfer->toreq, curfle->destfile, 
+                                     NULL, curfle->startsize);
       if (fromsize < 0)
         return (1);
 
-      gettimeofday (&starttime, NULL);
-      memcpy (&lasttime, &starttime, sizeof (lasttime));
+      gettimeofday (&transfer->starttime, NULL);
+      memcpy (&transfer->lasttime, &transfer->starttime, 
+              sizeof (transfer->lasttime));
       memset (&updatetime, 0, sizeof (updatetime));
   
       total = 0;
@@ -706,10 +784,11 @@ gftp_text_transfer_files (gftp_transfer * transfer)
 
           total += num_read;
           gftp_text_calc_kbs (transfer, num_read);
-          if (lasttime.tv_sec - updatetime.tv_sec >= 1 || total >= fromsize)
+          if (transfer->lasttime.tv_sec - updatetime.tv_sec >= 1 || total >= fromsize)
             {
               sw = gftp_text_get_win_size () - 20;
               tot = (float) total / (float) fromsize * (float) sw;
+                        
               if (tot > sw)
                 tot = sw;
               printf ("[");
@@ -719,7 +798,7 @@ gftp_text_transfer_files (gftp_transfer * transfer)
                 printf (" ");
               printf ("] @ %.2fKB/s", transfer->kbs);
               fflush (stdout);
-              memcpy (&updatetime, &lasttime, sizeof (updatetime));
+              memcpy (&updatetime, &transfer->lasttime, sizeof (updatetime));
             }
 
           if (GFTP_GET_DATA_TYPE (transfer->fromreq) == GFTP_TYPE_ASCII)
@@ -744,7 +823,7 @@ gftp_text_transfer_files (gftp_transfer * transfer)
       if (num_read < 0)
         {
           gftp_text_log (gftp_logging_misc, NULL, 
-                         _("Could not download %s from %s\n"), curfle->file);
+                         _("Could not download %s\n"), curfle->file);
           gftp_disconnect (transfer->fromreq);
           gftp_disconnect (transfer->toreq);
         }
@@ -863,41 +942,53 @@ gftp_text_get_win_size (void)
 void
 gftp_text_calc_kbs (gftp_transfer * tdata, ssize_t num_read)
 {
-  float difftime, curkbs;
+  double difftime, curkbs;
   unsigned long waitusecs;
+  gftp_file * tempfle;
   struct timeval tv;
+  unsigned long toadd;
 
+  tempfle = tdata->curfle->data;
   tdata->trans_bytes += num_read;
   tdata->curtrans += num_read;
   tdata->stalled = 0;
 
   gettimeofday (&tv, NULL);
-  difftime = (tv.tv_sec - tdata->lasttime.tv_sec) + ((float) (tv.tv_usec -
-             tdata->lasttime.tv_usec) / 1000000);
-
+  difftime = (tv.tv_sec - tdata->starttime.tv_sec) + ((double) (tv.tv_usec -
+             tdata->starttime.tv_usec) / 1000000);
   if (difftime == 0)
-    curkbs = (float) num_read / 1024;
+    tdata->kbs = (double) ((tdata->curtrans - tempfle->startsize) / 1024.0);
   else
-    curkbs = ((float) num_read / 1024) / difftime;
+    tdata->kbs = (double) (((tdata->curtrans - tempfle->startsize) / 1024.0) / difftime);
 
-  if (maxkbs > 0 && curkbs > maxkbs)
+  difftime = (double) ((tv.tv_sec - tdata->lasttime.tv_sec) + ((tv.tv_usec - tdata->lasttime.tv_usec) / 1000000));
+  if (difftime <= 0)
+    curkbs = (double) (num_read / 1024.0);
+  else
+    curkbs = (double) ((num_read / 1024.0) / difftime);
+
+  if (tdata->fromreq->maxkbs > 0 && curkbs > tdata->fromreq->maxkbs)
     {
-      waitusecs = ((float) num_read / 1024) / maxkbs * 1000000 - difftime;
+      waitusecs = (double) ((num_read / 1024.0) / tdata->fromreq->maxkbs * 1000000.0 - difftime);
+      difftime += (waitusecs / 1000000.0);
       usleep (waitusecs);
     }
-  else
-    waitusecs = 0;
 
-  difftime += waitusecs;
-  if (difftime == 0)
-    tdata->kbs = 0;
-  else
-    tdata->kbs = ((float) num_read / 1024) / (float) difftime;
-  memcpy (&tdata->lasttime, &tv, sizeof (tdata->lasttime));
+  difftime += tdata->lasttime.tv_usec / 1000000.0;
+  toadd = (long) difftime;
+  difftime -= toadd;
+  tdata->lasttime.tv_sec += toadd;
+  tdata->lasttime.tv_usec = difftime * 1000000.0;
 }
 
 
-#ifndef HAVE_GETADDRINFO
+void
+sig_child (int signo)
+{
+}
+
+
+#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
 
 struct hostent *
 r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
